#!/usr/bin/env python
# -*- coding:utf-8 -*-

"""
Basic tests for WPTools.
"""

import unittest
import wptools

from . import category
from . import category_cmcontinue
from . import disambiguation
from . import imageinfo
from . import labels_1
from . import labels_2
from . import labels_3
from . import labels_wanted
from . import parse
from . import parse_infobox
from . import parse_62
from . import parse_66
from . import parse_91
from . import parse_109
from . import query
from . import query_plcontinue
from . import querymore
from . import querymore_blcontinue
from . import random_query
from . import redirect
from . import rest_page
from . import rest_html
from . import rest_lead
from . import rest_summary
from . import siteinfo
from . import sitematrix
from . import siteviews
from . import wikidata
from . import wikidata_deleted
from . import wikidata_novalue_snak

SILENT_FLAG = True
SKIP_FLAG = ['imageinfo', 'labels']


class WPToolsTestCase(unittest.TestCase):

    @staticmethod
    def test_entry_points():
        wptools.core
        wptools.query
        wptools.request
        wptools.utils

        wptools.page
        wptools.category
        wptools.restbase
        wptools.site
        wptools.wikidata


class WPToolsCategoryTestCase(unittest.TestCase):

    def test_category_init(self):
        self.assertRaises(ValueError, wptools.category, pageid='TEST')
        self.assertRaises(ValueError, wptools.category, 'TEST', pageid=123)

        cat = wptools.category('TEST')
        self.assertEqual(cat.params, {'lang': 'en', 'title': 'TEST'})
        self.assertTrue('requests' not in cat.data)

        try:
            cat = wptools.category(namespace='NOTINT')
            self.fail("failed to raise ValueError")
        except ValueError:
            pass

    def test_category_random(self):
        cat = wptools.category('TEST')
        cat.cache = {'random': random_query.cache}
        cat._set_data('random')
        self.assertEqual(cat.data['title'], 'RANDOM TEST TITLE')

    def test_category_caching(self):
        cat = wptools.category('TEST', silent=SILENT_FLAG)
        cat.cache['category'] = {'response': None}
        cat.get_members()
        self.assertEqual(len(cat.data), 0)
        self.assertTrue('requests' not in cat.data)

    def test_category_get_members(self):
        cat = wptools.category('TEST')
        cat.cache['category'] = category.cache
        cat._set_data('category')
        self.assertEqual(len(cat.data['members']), 68)
        self.assertEqual(len(cat.data['subcategories']), 24)
        self.assertTrue('requests' not in cat.data)

    def test_category_get_members_namespace(self):
        cat = wptools.category('TEST', namespace=0)
        cat.cache['category'] = category.cache
        cat._set_data('category')
        self.assertEqual(len(cat.data['members']), 68)
        self.assertTrue('requests' not in cat.data)

    def test_category_get_members_continue(self):
        cat = wptools.category('TEST')
        cat.cache['category'] = category_cmcontinue.cache
        cat._set_data('category')
        self.assertTrue('continue' in cat.data)
        self.assertEqual(len(cat.data['members']), 1)

        qry = cat._query('category', wptools.query.WPToolsQuery())
        self.assertTrue('&cmcontinue=page|' in qry)
        self.assertTrue(qry.endswith('|42525291'))

        cat.cache['category'] = category.cache
        cat._set_data('category')
        self.assertTrue('cmcontinue' not in cat.data)
        self.assertEqual(len(cat.data['members']), 69)

    def test_category_query(self):
        cat = wptools.category('TEST')
        qobj = wptools.query.WPToolsQuery()
        self.assertEqual(cat._query('random', qobj),
                         ('https://en.wikipedia.org/w/api.php?'
                          'action=query&format=json&formatversion=2'
                          '&list=random&rnlimit=1&rnnamespace=14'))
        self.assertEqual(cat._query('category', qobj),
                         ('https://en.wikipedia.org/w/api.php?'
                          'action=query&format=json&formatversion=2'
                          '&list=categorymembers&cmlimit=500&cmtitle=TEST'))
        self.assertTrue('requests' not in cat.data)


class WPToolsCoreTestCase(unittest.TestCase):

    def test_core_not_implemented(self):

        class TEST(wptools.core.WPTools):
            def __init__(self):
                pass

        page = TEST()

        try:
            page._query(None, None)
            self.fail("failed to raise NotImplementedError")
        except NotImplementedError:
            pass

        try:
            page._set_data(None)
            self.fail("failed to raise NotImplementedError")
        except NotImplementedError:
            pass

    def test_core_load_missing(self):
        page = wptools.page('TEST')

        # empty response
        page.cache = {'query': {'query': 'QUERY', 'response': ''}}
        self.assertRaises(ValueError, page._load_response, 'query')

        # bad JSON
        page.cache = {'query': {'query': 'QUERY', 'response': '{TEST}'}}
        self.assertRaises(ValueError, page._load_response, 'query')

        # warnings
        page.cache = {
            'query':
            {'query': 'QUERY',
             'response': '{"warnings":{"query":{ "warnings":"TEST"}}}'}}
        page._load_response('query')

        # API error
        page.cache = {'parse':
                      {'query': 'QUERY',
                       'response': '{"error": {"code": "TEST"}}'}}
        self.assertRaises(LookupError, page._load_response, 'parse')

        # missing parse
        page.cache = {'parse': {'query': 'QUERY', 'response': '{}'}}
        self.assertRaises(LookupError, page._load_response, 'parse')

        # missing query
        page.cache = {'query':
                      {'query': 'QUERY',
                       'response': '{"query":{"pages":[{"missing": true}]}}'}}
        self.assertRaises(LookupError, page._load_response, 'query')

        # missing wikidata
        page.cache = {'wikidata':
                      {'query': 'QUERY',
                       'response': '{"entities":{"-1": {"missing": ""}}}'}}
        self.assertRaises(LookupError, page._load_response, 'wikidata')

        # show
        page.cache = {'query': query.cache}
        page.flags['skip'] = SKIP_FLAG
        page._set_data('query')
        page.show()
        self.assertTrue('requests' not in page.data)

    def test_core_info(self):
        page = wptools.page('TEST')
        page.cache = {'query': query.cache}
        self.assertEqual(list(page.info()), ['query'])
        self.assertEqual(page.info('query')['status'], 200)
        self.assertTrue('requests' not in page.data)

    def test_core_query(self):
        page = wptools.page('TEST')
        page.cache = {'query': query.cache}
        self.assertEqual(list(page.query()), ['query'])
        qstr = page.query('query')
        start = 'https://en.wikipedia.org/w/api.php?action=query'
        end = '&titles=Douglas%20Adams'
        self.assertTrue(qstr.startswith(start))
        self.assertTrue(qstr.endswith(end))
        self.assertTrue('requests' not in page.data)

    def test_core_response(self):
        page = wptools.page('TEST')
        page.cache = {'query': query.cache}
        self.assertEqual(list(page.response()), ['query'])
        self.assertTrue('query' in page.response('query'))
        self.assertTrue('requests' not in page.data)

    def test_core_safestr(self):
        self.assertEqual(wptools.core.safestr(None), None)
        self.assertEqual(wptools.core.safestr(1), '1')
        self.assertTrue(isinstance(wptools.core.safestr(u'ü'), str))

    def test_core_get(self):
        page = wptools.page('TEST')
        try:
            page._get('TEST', False, None, 0)
            self.fail("failed to raise ValueError")
        except ValueError:
            pass

    def test_core_request_limit(self):
        page = wptools.page('TEST')
        page.REQUEST_LIMIT = 0
        page.data['requests'] = ['TEST']
        try:
            page._get('TEST', False, None, 0)
            self.fail("failed to raise StopIteration")
        except StopIteration:
            pass

    def test_core_show(self):
        page = wptools.page('TEST')
        page.data['TEST1'] = None
        page.data['TEST2'] = tuple(range(3))
        page.show()

    def test_core_get_random(self):
        # MAKE ACTUAL REQUEST! (for to maximize coverage)
        page = wptools.page('TEST')
        page._get('random', True, None, 0)
        self.assertTrue('random' in page.cache)
        self.assertTrue('pageid' in page.data)


class WPToolsPageTestCase(unittest.TestCase):

    def test_core_init(self):
        page = wptools.page('TEST', skip='SKIP')
        self.assertEqual(page.flags,
                         {'silent': False, 'skip': 'SKIP', 'verbose': False})

        page = wptools.page('TEST', variant='VARIANT')
        self.assertEqual(page.params,
                         {'lang': 'en', 'title': 'TEST', 'variant': 'VARIANT'})

        page = wptools.page('TEST', wiki='WIKI')
        self.assertEqual(page.params,
                         {'lang': 'en', 'title': 'TEST', 'wiki': 'WIKI'})

    def test_page_init(self):
        page = wptools.page('TEST', silent=True)
        self.assertEqual(page.params, {'lang': 'en', 'title': 'TEST'})
        self.assertEqual(page.flags, {'silent': True, 'verbose': False})

        page = wptools.page(pageid=123, silent=SILENT_FLAG)
        self.assertEqual(page.params, {'lang': 'en', 'pageid': 123})

        page = wptools.page(wikibase='Q42', silent=SILENT_FLAG)
        self.assertEqual(page.params, {'lang': 'en', 'wikibase': 'Q42'})

        self.assertTrue('requests' not in page.data)

    def test_page_caching(self):
        page = wptools.page('TEST', silent=SILENT_FLAG)

        page.cache['parse'] = {'response': None}
        page.cache['query'] = {'response': None}
        page.cache['restbase'] = {'response': None}
        page.cache['wikidata'] = {'response': None}

        page.get_parse()
        page.get_query()
        page.get_restbase()
        page.get_wikidata()

        self.assertEqual(len(page.data), 0)
        self.assertTrue('requests' not in page.data)

    def test_page_query(self):
        page = wptools.page('TEST')
        qobj = wptools.query.WPToolsQuery()

        qstr = page._query('random', qobj)
        self.assertTrue('list=random' in qstr)

        qstr = page._query('query', qobj)
        self.assertTrue('action=query' in qstr)
        self.assertTrue('pageprops' in qstr)

        qstr = page._query('querymore', qobj)
        self.assertTrue('action=query' in qstr)
        self.assertTrue('&bllimit=500' in qstr)
        self.assertTrue('&cllimit=500' in qstr)
        self.assertTrue('&imlimit=500' in qstr)
        self.assertTrue('&lllimit=500&' in qstr)
        self.assertTrue('&pclimit=500' in qstr)

        qstr = page._query('parse', qobj)
        self.assertTrue('action=parse' in qstr)
        self.assertTrue('parsetree' in qstr)

        page.data['entities'] = ['Q1', 'Q2', 'Q3']
        qstr = page._query('labels', qobj)
        self.assertTrue('action=wbgetentities' in qstr)
        self.assertTrue('ids=Q1|Q2|Q3' in qstr)

        qstr = page._query('wikidata', qobj)
        self.assertTrue('action=wbgetentities' in qstr)

        page.params.update({'rest_endpoint': '/page/summary/'})
        qstr = page._query('restbase', qobj)
        self.assertTrue('api/rest' in qstr)

        self.assertTrue('requests' not in page.data)

    def test_page_get(self):
        """
        test page.get() without making any requests, is for coverage
        """
        wptools.request.WPToolsRequest.DISABLED = True
        skip = ['parse', 'query', 'restbase', 'wikiata']

        page = wptools.page('TEST', skip=skip)
        page.get()

        page = wptools.page('TEST', wikibase='TEST', skip=skip)
        page.get()

    def test_page_get_parse(self):
        page = wptools.page('TEST', skip=SKIP_FLAG, silent=SILENT_FLAG)
        page.cache = {'parse': parse.cache}
        page._set_data('parse')
        data = page.data
        self.assertEqual(data['pageid'], 8091)
        self.assertEqual(len(data['infobox']), 13)
        self.assertEqual(len(data['iwlinks']), 2)
        self.assertEqual(len(data['parsetree']), 78594)
        self.assertEqual(len(data['wikitext']), 65836)
        self.assertEqual(str(data['title']), 'Douglas Adams')
        self.assertEqual(str(data['wikibase']), 'Q42')
        self.assertTrue('satire' in data['infobox']['genre'])
        self.assertTrue(data['wikidata_url'].startswith('http'))
        self.assertEqual(str(data['image'][0]['file']),
                         'File:Douglas adams portrait cropped.jpg')

        self.assertTrue('requests' not in page.data)


    def test_page_get_parse_infobox(self):
        """
        Ensure cover images are captured from infobox
        """
        page = wptools.page('TEST', skip=SKIP_FLAG, silent=SILENT_FLAG)
        page.cache = {'parse': parse_infobox.cache}
        page._set_data('parse')
        data = page.data
        self.assertEqual(len(data['infobox']), 16)
        self.assertEqual(len(page.images()), 1)
        self.assertEqual(page.images('kind'),
                         [{'kind': 'parse-cover'}])
        self.assertEqual(page.images()[0]['file'],
                         u'File:John Coltrane - Blue Train.jpg')
        self.assertTrue('requests' not in page.data)


    def test_page_get_parse_62(self):
        """
        Get infobox data with list values. Issue #62
        """
        page = wptools.page('Lewisit', lang='de',
                            skip=SKIP_FLAG, silent=SILENT_FLAG)
        page.cache = {'parse': parse_62.cache}
        page._set_data('parse')
        infobox = page.data['infobox']
        self.assertEqual(len(infobox['Dichte'].split('*')), 3)
        self.assertEqual(len(infobox['Schmelzpunkt'].split('*')), 3)
        self.assertEqual(len(infobox['Siedepunkt'].split('*')), 3)
        self.assertEqual(len(infobox['Brechungsindex'].split('*')), 3)
        self.assertEqual(len(infobox['ToxDaten'].split('*')), 3)
        self.assertEqual(len(infobox['Andere Namen'].split('*')), 6)
        self.assertTrue('requests' not in page.data)

    def test_page_get_parse_66(self):
        """
        Get infobox data with sub-elements. Issue #66
        """
        page = wptools.page("ONE OK ROCK", lang='ja',
                            skip=SKIP_FLAG, silent=SILENT_FLAG)
        page.cache = {'parse': parse_66.cache}
        page._set_data('parse')
        infobox = page.data['infobox']
        self.assertEqual(len(infobox['Genre'].split('<br')), 8)
        self.assertTrue('requests' not in page.data)

    def test_page_get_parse_91(self):
        """
        Get infobox data with unusual wikitext syntax
        """
        page = wptools.page('Okapi', lang='fr',
                            skip=SKIP_FLAG, silent=SILENT_FLAG)
        page.cache = {'parse': parse_91.cache}
        page._set_data('parse')
        infobox = page.data['infobox']
        self.assertEqual(len(infobox['boxes']), 13)
        self.assertTrue(u'Taxobox début' in page.data['infobox']['boxes'][0])
        self.assertEqual(list(infobox['boxes'][9].keys()), ['Taxobox UICN'])
        self.assertTrue('requests' not in page.data)

    def test_page_get_parse_109(self):
        """
        Get infobox data with mix of links, templates, tail text
        """
        page = wptools.page('Orléans-cléry', lang='fr',
                            skip=SKIP_FLAG, silent=SILENT_FLAG)
        page.cache = {'parse': parse_109.cache}
        page._set_data('parse')
        infobox = page.data['infobox']
        self.assertTrue('[[cabernet franc]]' in infobox[u'cépages'])
        self.assertEqual(infobox[u'volproduction'],
                         u'{{unité|848|hectolitres}} en [[2009]]')
        self.assertEqual(infobox[u'densité'],
                         u'minimum de {{unité|5000|pieds}} par hectare')
        self.assertTrue('requests' not in page.data)

    def test_page_get_parse_boxterm(self):
        page = wptools.page('TEST', boxterm='TEST',
                            skip=SKIP_FLAG, silent=SILENT_FLAG)
        page.cache = {'parse': parse.cache}
        page._set_data('parse')
        self.assertTrue(page.data['infobox'] is None)
        self.assertTrue('requests' not in page.data)

    def test_page_get_query(self):
        page = wptools.page('TEST', skip=SKIP_FLAG, silent=SILENT_FLAG)
        page.cache = {'query': query.cache}
        page._set_data('query')
        data = page.data
        self.assertEqual(data['description'], u'author')
        self.assertEqual(data['label'], 'Douglas Adams')
        self.assertEqual(data['length'], 62123)
        self.assertEqual(data['pageid'], 8091)
        self.assertEqual(data['watchers'], 460)
        self.assertEqual(len(data['aliases']), 3)
        self.assertEqual(len(data['assessments']), 10)
        self.assertEqual(len(data['image']), 2)
        self.assertEqual(len(data['links']), 384)
        self.assertEqual(str(data['wikibase']), 'Q42')
        self.assertTrue('page' in data['modified'])
        self.assertTrue(data['extext'].startswith('**Douglas'))
        self.assertTrue(data['extract'].startswith('<p><b>Douglas'))
        self.assertTrue(data['url'].endswith('Douglas_Adams'))
        self.assertTrue(data['url_raw'].endswith('_Adams?action=raw'))
        self.assertTrue(data['wikidata_url'].endswith('Q42'))
        self.assertTrue(wptools.utils.is_text(page.data['random']))

        self.assertTrue('requests' not in page.data)

    def test_page_get_more(self):
        page = wptools.page('TEST', silent=SILENT_FLAG)
        page.cache = {'querymore': querymore.cache}
        page.get_more()
        page._set_data('querymore')
        self.assertEqual(len(page.data['backlinks']), 500)
        self.assertTrue(len(page.data['categories']), 29)
        self.assertTrue(len(page.data['files']), 12)
        self.assertTrue(len(page.data['languages']), 70)
        self.assertTrue(page.data['contributors'], 1317)
        self.assertTrue(page.data['views'], 1398)

        self.assertTrue('requests' not in page.data)

    def test_page_get_query_continue(self):
        page = wptools.page('TEST', skip=SKIP_FLAG, silent=SILENT_FLAG)

        page.cache['query'] = query.cache
        page._set_data('query')
        self.assertEqual(len(page.data['links']), 384)

        page.cache = {'query': query_plcontinue.cache}
        page._set_data('query')
        self.assertTrue('continue' in page.data)
        self.assertEqual(page.data['continue'],
                         {'plcontinue': u'144146|0|Perl_Data_Language'})
        self.assertEqual(len(page.data['links']), 385)

        qry = page._query('query', wptools.query.WPToolsQuery())
        self.assertTrue(qry.endswith('&plcontinue=144146|0|Perl_Data_Language'))

    def test_page_get_more_continue(self):
        page = wptools.page('TEST', skip=SKIP_FLAG, silent=SILENT_FLAG)

        page.cache['querymore'] = querymore.cache
        page._set_data('querymore')
        self.assertEqual(len(page.data['backlinks']), 500)
        self.assertEqual(page.data['continue'], {'blcontinue': u'4|2028922'})

        page.cache = {'querymore': querymore_blcontinue.cache}
        page._set_data('querymore')
        self.assertTrue('continue' in page.data)
        self.assertEqual(len(page.data['backlinks']), 501)

        qry = page._query('querymore', wptools.query.WPToolsQuery())
        self.assertTrue(qry.endswith('&blcontinue=0|2062757'))

    def test_page_get_imageinfo(self):
        page = wptools.page('TEST', silent=SILENT_FLAG)

        self.assertRaises(ValueError, page.get_imageinfo)

        page.cache = {'imageinfo': imageinfo.cache}
        page.data['image'] = [{'kind': 'parse-image',
                               'file': 'Douglas adams portrait cropped.jpg'}]

        page._normalize_images()
        query = page._query('imageinfo', wptools.query.WPToolsQuery())
        self.assertTrue('File%3ADouglas' in query)

        page._set_data('imageinfo')
        image = page.data['image'][0]
        self.assertTrue('/c/c0/' in image['url'])
        self.assertTrue('/commons.' in image['descriptionurl'])
        self.assertTrue(image['file'].startswith('File:'))
        self.assertEqual(image['height'], 386)
        self.assertEqual(image['size'], 32915)
        self.assertEqual(image['width'], 333)

        self.assertTrue('requests' not in page.data)

    def test_page_get_random(self):
        page = wptools.page('TEST', skip=SKIP_FLAG, silent=SILENT_FLAG)
        page.cache = {'random': query.cache}
        page._set_data('random')
        page.get_random()
        self.assertEqual(page.data['pageid'], 7614573)
        self.assertEqual(page.data['title'], u'Aidenbachstraße (Munich U-Bahn)')

        self.assertTrue('requests' not in page.data)

    def test_page_get_restbase(self):
        page = wptools.page('TEST', silent=SILENT_FLAG)
        page.params.update({'rest_endpoint': 'summary'})
        page.cache = {'restbase': rest_summary.cache}
        page._set_data('restbase')
        data = page.data
        self.assertEqual(data['pageid'], 8091)
        self.assertTrue(data['exhtml'].startswith("<p><b>Douglas"))
        self.assertTrue(data['exrest'].startswith("Douglas"))

        self.assertTrue('requests' not in page.data)

    def test_page_get_wikidata(self):
        page = wptools.page('TEST',
                            wikibase='WIKIBASE',
                            skip=SKIP_FLAG,
                            silent=SILENT_FLAG)

        page.cache = {'wikidata': wikidata.cache}
        page._set_data('wikidata')

        page.cache['labels'] = labels_1.cache
        page._set_data('labels')

        page.cache['labels'] = labels_2.cache
        page._set_data('labels')

        page.cache['labels'] = labels_3.cache
        page._set_data('labels')

        page._post_labels_updates()

        page.cache['imageinfo'] = imageinfo.cache
        page._set_data('imageinfo')

        data = page.data
        self.assertEqual(data['wikibase'], 'Q42')
        self.assertEqual(data['image'][0]['kind'], 'wikidata-image')
        self.assertEqual(len(data['claims']), 102)
        self.assertEqual(len(data['labels']), 147)
        self.assertEqual(len(data['wikidata']), 102)

        self.assertTrue('requests' not in page.data)

    def test_page_pageimage(self):
        page = wptools.page('TEST', silent=SILENT_FLAG)
        page.pageimage()
        page.data['image'] = [{'kind': 'TEST'}, {'kind': 'TESTMORE'}]
        page.pageimage()
        page.pageimage('MORE')

        self.assertTrue('requests' not in page.data)

    def test_page_get_disambiguation(self):
        page = wptools.page('TEST', silent=SILENT_FLAG)
        page.cache = {'query': disambiguation.cache}
        page._set_data('query')
        data = page.data
        self.assertEqual(data['disambiguation'], 10)
        self.assertEqual(len(data['links']), 10)

        self.assertTrue('requests' not in page.data)

    def test_page_get_redirect(self):
        page = wptools.page('TEST', skip=SKIP_FLAG, silent=SILENT_FLAG)
        page.cache = {'query': redirect.cache}
        page._set_data('query')
        data = page.data
        self.assertEqual(data['redirected'][0]['from'], u'Adams, Douglas')
        self.assertEqual(len(data['redirects']), 12)

        self.assertTrue('requests' not in page.data)


class WPToolsQueryTestCase(unittest.TestCase):

    def test_query_init(self):
        qobj = wptools.query.WPToolsQuery()
        self.assertEqual(qobj.lang, 'en')
        self.assertEqual(qobj.uri, 'https://en.wikipedia.org')
        self.assertEqual(qobj.wiki, 'en.wikipedia.org')

    def test_query_init_wiki(self):
        qobj = wptools.query.WPToolsQuery(wiki='http://example.com')
        self.assertEqual(qobj.domain, 'example.com')
        self.assertEqual(qobj.uri, 'http://example.com')

    def test_query_init_endpoint(self):
        qobj = wptools.query.WPToolsQuery(endpoint='ENDPOINT')
        self.assertEqual(qobj.endpoint, 'ENDPOINT')
        self.assertEqual(qobj.uri, 'https://en.wikipedia.org')
        self.assertTrue('.orgENDPOINT?' in qobj.category('CAT'))
        self.assertTrue('.orgENDPOINT?' in qobj.labels(['QID']))
        self.assertTrue('.orgENDPOINT?' in qobj.imageinfo(['FILE']))
        self.assertTrue('.orgENDPOINT?' in qobj.parse('TITLE'))
        self.assertTrue('.orgENDPOINT?' in qobj.query('TITLE'))
        self.assertTrue('.orgENDPOINT?' in qobj.querymore('TITLE'))
        self.assertTrue('.orgENDPOINT?' in qobj.site('siteinfo'))
        self.assertTrue('.orgENDPOINT?' in qobj.wikidata('TITLE'))

    def test_query_init_lang(self):
        qobj = wptools.query.WPToolsQuery(lang='zz')
        self.assertEqual(qobj.lang, 'zz')
        self.assertEqual(qobj.domain, 'zz.wikipedia.org')
        self.assertEqual(qobj.uri, 'https://zz.wikipedia.org')

    def test_query_category(self):
        qobj = wptools.query.WPToolsQuery()

        qstr = qobj.category(title='TEST')
        self.assertTrue(qstr.startswith('https://en.wikipedia.org'))
        self.assertTrue('&list=categorymembers' in qstr)
        self.assertTrue('&cmtitle=TEST' in qstr)
        self.assertTrue('&cmpageid' not in qstr)
        self.assertEqual(qobj.status,
                         'en.wikipedia.org (categorymembers) TEST')

        qstr = qobj.category(None, pageid=123)
        self.assertTrue(qstr.startswith('https://en.wikipedia.org'))
        self.assertTrue('&list=categorymembers' in qstr)
        self.assertTrue('&cmpageid=123' in qstr)
        self.assertTrue('&cmtitle' not in qstr)
        self.assertEqual(qobj.status,
                         'en.wikipedia.org (categorymembers) 123')

    def test_query_labels(self):
        qobj = wptools.query.WPToolsQuery()
        qstr = qobj.labels(qids=['Q1', 'Q2', 'Q3'])
        self.assertTrue(qstr.startswith('https://www.wikidata.org'))
        self.assertTrue('?action=wbgetentities' in qstr)
        self.assertTrue('&ids=Q1|Q2|Q3' in qstr)
        self.assertEqual(qobj.status, 'www.wikidata.org (labels) Q1|Q2|Q3')

    def test_query_domain_name(self):
        domain = wptools.query.domain_name('http://example.com/a//b/c/')
        self.assertEqual(domain, 'example.com')

    def test_query_imageinfo(self):
        qobj = wptools.query.WPToolsQuery()
        qstr = qobj.imageinfo(files=['A', 'B', 'C'])
        self.assertTrue(qstr.startswith('https://en.wikipedia.org'))
        self.assertTrue('?action=query' in qstr)
        self.assertTrue('&prop=imageinfo' in qstr)
        self.assertTrue('&titles=A|B|C' in qstr)
        self.assertEqual(qobj.status, 'en.wikipedia.org (imageinfo) A|B|C')

    def test_query_query(self):
        qobj = wptools.query.WPToolsQuery()

        qstr = qobj.query(titles='TEST')
        self.assertTrue(qstr.startswith('https://en.wikipedia.org'))
        self.assertTrue('?action=query' in qstr)
        self.assertTrue('&titles=TEST' in qstr)
        self.assertEqual(qobj.status, 'en.wikipedia.org (query) TEST')

        qstr = qobj.query(None, pageids=123)
        self.assertTrue(qstr.startswith('https://en.wikipedia.org'))
        self.assertTrue('?action=query' in qstr)
        self.assertTrue('&pageids=123' in qstr)
        self.assertEqual(qobj.status, 'en.wikipedia.org (query) 123')

    def test_query_parse(self):
        qobj = wptools.query.WPToolsQuery()

        qstr = qobj.parse(title='TEST')
        self.assertTrue(qstr.startswith('https://en.wikipedia.org'))
        self.assertTrue('?action=parse' in qstr)
        self.assertTrue('&page=TEST' in qstr)
        self.assertEqual(qobj.status, 'en.wikipedia.org (parse) TEST')

        qstr = qobj.parse(None, pageid=123)
        self.assertTrue(qstr.startswith('https://en.wikipedia.org'))
        self.assertTrue('?action=parse' in qstr)
        self.assertTrue('&redirects' not in qstr)
        self.assertTrue('&pageid=123' in qstr)
        self.assertEqual(qobj.status, 'en.wikipedia.org (parse) 123')

    def test_query_random(self):
        qobj = wptools.query.WPToolsQuery()
        qstr = qobj.random()
        self.assertTrue(qstr.startswith('https://en.wikipedia.org'))
        self.assertTrue('?action=query' in qstr)
        self.assertTrue('&list=random' in qstr)
        self.assertTrue('en.wikipedia.org (random)' in qobj.status)

    def test_query_rest(self):
        qobj = wptools.query.WPToolsQuery()
        qstr = qobj.restbase(endpoint='/TEST/', title='TEST')
        self.assertTrue(qstr.startswith('https://en.wikipedia.org'))
        self.assertTrue('/api/rest' in qstr)
        self.assertTrue(qstr.endswith('/TEST/TEST'))
        self.assertEqual(qobj.status, 'en.wikipedia.org (restbase) /TEST/TEST')

    def test_query_set_status(self):
        qobj = wptools.query.WPToolsQuery()
        qobj.set_status('TEST_ACTION', 'TEST_TARGET')
        self.assertEqual(qobj.status,
                         'en.wikipedia.org (TEST_ACTION) TEST_TARGET')
        qobj.set_status('--------------------------------------------------',
                        '--------------------------------------------------')
        self.assertEqual(len(qobj.status), 68)

    def test_query_wiki_uri(self):
        qobj = wptools.query.WPToolsQuery()
        uri = qobj.wiki_uri(wiki='http://example.com/')
        self.assertEqual(uri, 'http://example.com/')

    def test_query_wikidata(self):
        qobj = wptools.query.WPToolsQuery()

        qstr = qobj.wikidata(title='TEST')
        self.assertTrue(qstr.startswith('https://www.wikidata.org'))
        self.assertTrue('?action=wbgetentities' in qstr)
        self.assertTrue('&sites=enwiki' in qstr)
        self.assertTrue('&titles=TEST' in qstr)
        self.assertEqual(qobj.status, 'www.wikidata.org (wikidata) TEST')

        qstr = qobj.wikidata(None, wikibase='Q1')
        self.assertTrue(qstr.startswith('https://www.wikidata.org'))
        self.assertTrue('?action=wbgetentities' in qstr)
        self.assertTrue('&sites' not in qstr)
        self.assertTrue('&titles' not in qstr)
        self.assertTrue('&ids=Q1' in qstr)
        self.assertEqual(qobj.status, 'www.wikidata.org (wikidata) Q1')

    def test_query_variant_parse(self):
        qobj = wptools.query.WPToolsQuery(variant='zh-cn')
        qstr = qobj.parse('TEST')
        self.assertTrue('&variant=zh-cn' in qstr)

    def test_query_variant_query(self):
        qobj = wptools.query.WPToolsQuery(variant='zh-cn')
        qstr = qobj.query('TEST')
        self.assertTrue('&variant=zh-cn' in qstr)

    def test_query_variant_wikidata(self):
        qobj = wptools.query.WPToolsQuery(variant='zh-cn')
        qstr = qobj.wikidata('TEST')
        self.assertTrue('&languages=zh-cn' in qstr)


class WPToolsRESTBaseTestCase(unittest.TestCase):

    def test_restbase_init(self):
        page = wptools.restbase()
        self.assertEqual(page.params, {'lang': 'en'})
        self.assertTrue('requests' not in page.data)

    def test_get_restbase_page(self):
        page = wptools.restbase('/page/', silent=SILENT_FLAG)
        page.cache['restbase'] = rest_page.cache
        page._set_data('restbase')
        self.assertTrue('requests' not in page.data)

    def test_get_restbase_html(self):
        page = wptools.restbase('TEST', silent=SILENT_FLAG)
        page.cache['restbase'] = rest_html.cache
        page._set_data('restbase')

        html = page.data['html']

        self.assertTrue(len(html), 264870)
        self.assertTrue(html.startswith('<!DOCTYPE'))
        self.assertTrue(html.endswith('</html>'))

        self.assertTrue('requests' not in page.data)

    def test_get_restbase_lead(self):
        page = wptools.restbase('TEST', silent=SILENT_FLAG)
        page.cache['restbase'] = rest_lead.cache
        page._set_data('restbase')

        params = page.params
        data = page.data

        self.assertEqual(data['description'], 'English writer and humorist')
        self.assertEqual(data['pageid'], 8091)
        self.assertEqual(len(data['lead']), 6577)
        self.assertEqual(params['title'], 'TEST')
        self.assertEqual(str(data['title']), 'Douglas_Adams')
        self.assertEqual(str(data['wikibase']), 'Q42')
        self.assertTrue('page' in data['modified'])
        self.assertTrue(data['lead'].startswith('<span'))
        self.assertTrue(len(data['image']), 1)

        self.assertTrue('requests' not in page.data)

    def test_get_restbase_summary(self):
        page = wptools.restbase('TEST', silent=SILENT_FLAG)
        page.params.update({'rest_endpoint': '/page/summary/'})
        page.cache['restbase'] = rest_summary.cache
        page._set_data('restbase')

        data = page.data

        self.assertEqual(len(data['image']), 2)
        self.assertEqual(data['description'], 'English writer and humorist')
        self.assertEqual(data['pageid'], 8091)
        self.assertEqual(str(data['title']), 'Douglas_Adams')
        self.assertEqual(len(data['exhtml']), 967)
        self.assertEqual(len(data['exrest']), 894)
        self.assertTrue(data['exhtml'].startswith('<p>'))
        self.assertTrue(data['exrest'].startswith('Douglas'))

        self.assertTrue('requests' not in page.data)


class WPToolsRequestTestCase(unittest.TestCase):

    def test_request_init(self):
        req = wptools.request.WPToolsRequest()
        self.assertEqual(req.silent, False)
        self.assertEqual(req.verbose, False)
        self.assertTrue(isinstance(req.cobj, wptools.request.pycurl.Curl))

    def test_request_curl_info(self):
        req = wptools.request.WPToolsRequest()
        info = wptools.request.curl_info(req.cobj)
        self.assertEqual(info.get('status'), 0)
        self.assertEqual(info.get('bytes'), 0.0)
        self.assertEqual(info.get('content-type'), None)
        self.assertTrue('wptools' in info.get('user-agent'))

    def test_request_curl_setup(self):
        req = wptools.request.WPToolsRequest(verbose=True)
        req.curl_setup(proxy='TEST_PROXY', timeout=666)
        info = req.cobj.getinfo(wptools.request.pycurl.EFFECTIVE_URL)
        self.assertEqual(info, '')
        info = req.cobj.getinfo(wptools.request.pycurl.RESPONSE_CODE)
        self.assertEqual(info, 0)

    def test_request_curl_proxy_setup(self):
        req = wptools.request.WPToolsRequest(verbose=True)
        test_proxy = {
            'PROXY': 'TEST_PROXY',
            'PORT': 0,
            'USERPWD': 'TEST_USER:PWD'
        }
        req.curl_setup(proxy=test_proxy)

    def test_request_user_agent(self):
        agent = wptools.request.user_agent()
        self.assertTrue(agent.startswith('wptools'))
        self.assertTrue('(https://github.com/siznax/wptools)' in agent)

    def test_request_get(self):
        req = wptools.request.WPToolsRequest()
        try:
            req.get('TEST_URL', 'TEST_STATUS')
            self.fail("failed to raise pycurl error")
        except:
            pass


class WPToolsSiteTestCase(unittest.TestCase):

    def test_site_init(self):
        site = wptools.site()
        self.assertEqual(site.params, {'lang': 'en'})
        self.assertEqual(site.flags, {'silent': False, 'verbose': False})

    def test_site_query(self):
        site = wptools.site()

        qobj = wptools.query.WPToolsQuery(wiki='commons.wikimedia.org')
        query = site._query('sitematrix', qobj)
        self.assertTrue(query.startswith('https://commons.wikimedia.org'))

        qobj = wptools.query.WPToolsQuery()
        query = site._query('siteinfo', qobj)
        self.assertTrue('&meta=siteinfo|siteviews' in query)
        self.assertTrue('&siprop=general|statistics' in query)
        self.assertTrue('&list=mostviewed&pvimlimit=max' in query)
        query = site._query('sitevisitors', qobj)
        self.assertTrue('&meta=siteviews&pvismetric=uniques' in query)

        self.assertTrue('requests' not in site.data)

    def test_site_get_sites(self):
        site = wptools.site(silent=SILENT_FLAG)
        site.cache = {'sitematrix': sitematrix.cache}

        site.get_sites()
        site._set_data('sitematrix')
        self.assertEqual(len(site.data['sites']), 741)
        self.assertTrue(site.data.get('random') is not None)

        site.get_sites(domain='wikipedia.org')
        site._set_data('sitematrix')
        self.assertEqual(len(site.data['sites']), 290)

        # filter by domain
        site.params.update({'domain': 'wikipedia.org'})
        site._set_data('sitematrix')
        data = site.data
        self.assertEqual(len(data['sites']), 290)

        self.assertTrue('requests' not in site.data)

    def test_site_get_siteinfo(self):
        site = wptools.site(silent=SILENT_FLAG)
        site.cache = {'siteinfo': siteinfo.cache,
                      'sitevisitors': siteviews.cache}
        site.get_info(wiki='en.wikipedia.org')
        site._set_data('siteinfo')
        site._set_data('sitevisitors')
        data = site.data
        self.assertEqual(data['activeusers'], 126428)
        self.assertEqual(data['admins'], 1248)
        self.assertEqual(data['articles'], 5478623)
        self.assertEqual(data['edits'], 910438071)
        self.assertEqual(data['images'], 854622)
        self.assertEqual(len(data['info']), 49)
        self.assertEqual(data['pages'], 43112467)
        self.assertEqual(data['site'], 'enwiki')
        self.assertEqual(data['users'], 31786628)

        self.assertEqual(len(data['mostviewed']), 478)
        self.assertEqual(data['siteviews'], 233991363)
        self.assertEqual(data['visitors'], 63079969)

        site.top()
        site.top(wiki='en.wikipedia.org', limit=10)

        self.assertTrue('requests' not in site.data)


class WPToolsWikidataTestCase(unittest.TestCase):

    def test_wikidata_init(self):
        page = wptools.wikidata('TEST', silent=SILENT_FLAG)
        self.assertEqual(page.params, {'lang': 'en', 'title': 'TEST'})

        page = wptools.wikidata(wikibase='Q42', silent=SILENT_FLAG)
        self.assertEqual(page.params, {'lang': 'en', 'wikibase': 'Q42'})

        page = wptools.wikidata()
        try:
            page.get_wikidata()
            self.fail("failed to raise LookupError")
        except LookupError:
            pass

        try:
            page.wanted_labels('TEST')
            self.fail("failed to raise ValueError")
        except ValueError:
            pass

        self.assertTrue('requests' not in page.data)

    def test_wikidata_get_labels(self):
        page = wptools.wikidata(skip=SKIP_FLAG,
                                silent=SILENT_FLAG)
        page.cache = {'labels':  labels_1.cache,
                      'wikidata': wikidata.cache}

        page._set_data('wikidata')
        page._set_data('labels')
        page._post_labels_updates()

        data = page.data
        self.assertEqual(len(data['claims']), 102)
        self.assertEqual(len(data['labels']), 50)
        self.assertEqual(len(data['wikidata']), 27)

        page.get_labels()  # No entities found

        self.assertTrue('requests' not in page.data)

    def test_wikidata_get_wikidata(self):
        page = wptools.wikidata(skip=SKIP_FLAG, silent=SILENT_FLAG)

        page.cache = {'wikidata': wikidata.cache}
        page._set_data('wikidata')

        page.cache['labels'] = labels_1.cache
        page._set_data('labels')
        page.cache['labels'] = labels_2.cache
        page._set_data('labels')
        page.cache['labels'] = labels_3.cache
        page._set_data('labels')

        page._post_labels_updates()

        page.cache['imageinfo'] = imageinfo.cache
        page._set_data('imageinfo')

        self.assertTrue('requests' not in page.data)

        data = page.data
        self.assertEqual(data['description'], u'English writer and humorist')
        self.assertEqual(data['label'], 'Douglas Adams')
        self.assertEqual(len(data['claims']), 102)
        self.assertEqual(len(data['labels']), 147)
        self.assertEqual(len(data['wikidata']), 102)
        self.assertEqual(str(data['title']), 'Douglas_Adams')
        self.assertEqual(str(data['what']), 'human')
        self.assertEqual(str(data['wikibase']), 'Q42')
        self.assertTrue('wikidata' in data['modified'])
        self.assertTrue(data['wikidata_url'].endswith('Q42'))

        self.assertTrue('requests' not in page.data)

    def test_wikidata_disambig(self):
        """
        Ensure title param is set given only wikibase
        """
        page = wptools.wikidata(skip=SKIP_FLAG, wikibase='TEST')
        page.cache = {'wikidata': wikidata.cache}
        page._set_wikidata()
        self.assertEqual(str(page.params['title']), 'Douglas_Adams')

        self.assertTrue('requests' not in page.data)

    def test_wikidata_get_wanted(self):
        page = wptools.wikidata('TEST', skip=SKIP_FLAG)
        page.wanted_labels(['P31', 'Q5'])

        page.cache = {'wikidata': wikidata.cache}
        page._set_data('wikidata')

        page.cache['labels'] = labels_wanted.cache
        page._set_data('labels')

        page._post_labels_updates()

        data = page.data
        self.assertEqual(len(data['wikidata']), 1)
        self.assertTrue('instance of (P31)' in data['wikidata'])

    def test_wikidata_query(self):
        page = wptools.wikidata('TEST')
        qobj = wptools.query.WPToolsQuery()

        page.data['entities'] = ['TEST']
        qry = page._query('labels', qobj)
        self.assertTrue(qry.endswith('TEST'))

        qry = page._query('wikidata', qobj)
        self.assertTrue(qry.endswith('TEST'))

    def test_wikidata_image(self):
        """
        ensure we don't add wikidata-image if file info extant
        """
        page = wptools.wikidata(skip=SKIP_FLAG, silent=SILENT_FLAG)

        page.cache = {'wikidata': wikidata.cache}
        page._set_data('wikidata')

        page.data['image'] = [
            {'file': 'Douglas adams portrait cropped.jpg'}]

        page.cache['labels'] = labels_2.cache  # P18 in here
        page._set_data('labels')
        page._post_labels_updates()

        self.assertTrue('requests' not in page.data)

        data = page.data
        self.assertTrue(len(data['image']), 1)
        self.assertTrue('wikidata-image' not in data['image'])

    def test_wikidata_missing(self):
        page = wptools.wikidata(skip=SKIP_FLAG, silent=SILENT_FLAG)

        page.cache = {'wikidata': wikidata.cache}
        page._set_data('wikidata')

        del page.data['claims']['P31']

        page.cache['labels'] = labels_wanted.cache
        page._set_data('labels')
        page._post_labels_updates()

        self.assertTrue('requests' not in page.data)

        data = page.data
        self.assertTrue('what' not in data['wikidata'])

    def test_wikidata_deleted(self):
        page = wptools.wikidata(skip=SKIP_FLAG, silent=SILENT_FLAG)

        page.cache = {'wikidata': wikidata_deleted.cache}

        try:
            page._set_data('wikidata')
            self.fail("failed to raise LookupError")
        except LookupError:
            pass

        self.assertTrue('requests' not in page.data)

    def test_wikidata_no_value_snak(self):
        page = wptools.wikidata(skip=SKIP_FLAG, silent=SILENT_FLAG)
        page.cache = {'wikidata': wikidata_novalue_snak.cache}
        page._set_data('wikidata')
        self.assertEqual(len(page.data['claims']), 2)


class WPToolsToolTestCase(unittest.TestCase):

    def test_wptool(self):
        from scripts.wptool import main
        from collections import namedtuple
        args = namedtuple('Args', ['H', 'l', 'n', 'q', 's', 't', 'v', 'w'])
        cli = {'H': False, 'l': 'en', 'n': False, 'q': True, 's': True,
               't': '', 'v': False, 'w': ''}
        main(args(**cli))


if __name__ == '__main__':
    unittest.main()



# -*- coding:utf-8 -*-

query = 'https://www.wikidata.org/w/api.php?action=wbgetentities&formatversion=2&languages=en&props=labels&redirects=yes&ids=P31|Q5'

response = r"""{
    "entities": {
        "P31": {
            "type": "property",
            "datatype": "wikibase-item",
            "id": "P31",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "instance of"
                }
            }
        },
        "Q5": {
            "type": "item",
            "id": "Q5",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "human"
                }
            }
        }
    },
    "success": 1
}"""

cache = {'info': {'status': 200}, 'query': query, 'response': response}



# -*- coding:utf-8 -*-

'''
selected titles by language for WPTools tests
'''

# animals
#     https://www.worldwildlife.org/species/directory?direction=desc&sort=extinction_status
# art
#     https://en.wikipedia.org/wiki/Category:Lists_of_works_of_art
# books
#     https://en.wikipedia.org/wiki/Man_Booker_Prize
#     https://en.wikipedia.org/wiki/Man_Booker_International_Prize
#     https://en.wikipedia.org/wiki/Pulitzer_Prize_for_Fiction
# music
#     https://en.wikipedia.org/wiki/Grammy_Award_for_Album_of_the_Year
# film
#     https://en.wikipedia.org/wiki/List_of_Academy_Award-winning_films
#     https://en.wikipedia.org/wiki/List_of_films_considered_the_best
# featured
#     https://en.wikipedia.org/wiki/Wikipedia:Today%27s_featured_article/Most_viewed
#     https://ja.wikipedia.org/wiki/Wikipedia:%E7%A7%80%E9%80%B8%E3%81%AA%E8%A8%98%E4%BA%8B
#     https://ru.wikipedia.org/wiki/%D0%92%D0%B8%D0%BA%D0%B8%D0%BF%D0%B5%D0%B4%D0%B8%D1%8F:%D0%98%D0%B7%D0%B1%D1%80%D0%B0%D0%BD%D0%BD%D1%8B%D0%B5_%D1%81%D1%82%D0%B0%D1%82%D1%8C%D0%B8
#     https://zh.wikipedia.org/wiki/Wikipedia:%E7%89%B9%E8%89%B2%E6%9D%A1%E7%9B%AE
# places
#     https://en.wikipedia.org/wiki/List_of_cities_proper_by_population

import random


class WPToolsTestTitles(object):
    """
    selected test titles
    - lang: en, ja, ru, zh
    - categories: animals, featured, people, places, things
    """

    def __init__(self):
        pass

    animals = {
        "en": {
            "Amur Leopard",
            "Black Rhino",
            "Bornean Orangutan",
            "Cross River Gorilla",
            "Hawksbill Turtle",
            "Javan Rhino",
            "Leatherback Turtle",
            "Mountain Gorilla",
            "Orangutan",
            "Pangolin",
            "Saola",
            "South China Tiger",
            "Sumatran Elephant",
            "Sumatran Orangutan",
            "Sumatran Rhino",
            "Sumatran Tiger",
            "Vaquita",
            "Western Lowland Gorilla",
        }
    }

    art = {
        "en": {
            "Eight Elvises",
            "Guennol Lioness",
            "Portrait of Dr. Gachet",
            "The Great Wave off Kanagawa",
            "The Scream",
            "The Sea Monster",
            "Knight, Death and the Devil",
            "Melencolia I",
        }
    }

    books = {
        "en": {
            "A Brief History of Seven Killings",
            "Bring Up the Bodies",
            "MaddAddam",
            "The Sympathizer",
            "The Vegetarian",
        }
    }

    film = {
        "en": {
            "Casablanca (film)",
            "Himala",
            "Raise the Red Lantern",
            "Seven Samurai",
            "Spotlight (film)",
        }
    }

    music = {
        "en": {
            "1989 (Taylor Swift album)",
            "Beauty Behind the Madness",
            "Giant Steps",
            "Kind of Blue",
            "To Pimp a Butterfly",
        }
    }

    # these are somewhat random, but should be high-quality articles
    featured = {
        "en": {
            "Franz Kafka",
            "Elizabeth II",
            "Barack Obama",
            "Pluto",
            "Shah Rukh Khan",
        },
        "ja": {
            "ノストラダムス",  # Nostrodamus
            "アンネ・フランク",  # Anne Frank
            "フィンセント・ファン・ゴッホ",  # Vincent van Gogh
            "アイザック・アシモフ",  # Isaac Asimov
            "チンドン屋",  # musical sandwichmen
        },
        "ru": {
            "Малер, Густав",  # Gustav Mahler
            "Сенеш, Хана",  # Hannah Szenes
            "Уильямс, Серена",  # Crime and Punishment
            "Бронте, Энн",  # Anne Bronte
            "Уолстонкрафт, Мэри",  # Mary Wollstonecraft
        },
        "zh": {
            "異特龍屬",  # allosaurus
            "兴圣教寺塔",  # Hing temple
            "玛丽·沃斯通克拉夫特",  # Mary Wollstonecraft
            "蒙山大佛",  # Monsanto Buddha
            "尼克松在中国 (歌剧)",  # Nixon in China (opera)
        },
    }

    people = {
        "en": {
            "Amelia Earhart",
            "Anne Frank",
            "Borges",
            "Eleanor of Aquitaine",
            "Elizabeth II",
            "Ella Fitzgerald",
            "Emily Dickinson",
            "Fela Kuti",
            "Flannery O'Connor",
            "Frida Kahlo",
            "Harriet Tubman",
            "Harriet Beecher Stowe",
            "Jane Goodall",
            "Jeanne d'Arc",
            "Malala Yousafzai",
            "Malcolm X",
            "Stephen Fry",
            "Simone de Beauvoir",
            "Shakespeare",
        },
        "ja": {
            "三島由紀夫",  # Yukio Mishima
            "北斎",  # Hokusai
            "穐吉敏子",  # Toshiko Akiyoshi
            "草薙素子",  # Major Motoko Kusanagi
            "紫式部",  # Murasaki Shikibu
            "村上春樹",  # Haruki Murakami
        },
        "ru": {
            "Анна Ахматова",  # Anna Akhmatova
            "Анна Павлова",  # Anna Pavlova
            "Антон Павлович Чехов",  # Anton Chekhov
            "Екатерина II",  # Catherine the Great
            "Фёдор Михайлович Достоевский",  # Fyodor Dostoyevsky
            "Валентина Владимировна Терешкова",  # Valentina Tereshkova
        },
        "zh": {
            "老子",  # Laozi
            "丹增嘉措",  # Dalai Lama 14
            "艾未未",  # Ai Weiwei
            "刘晓波",  # Liu Xiaobo
            "楊紫瓊",  # Michelle Yeoh
            "刘洋 (航天员)",  # Liu Yang (astronaut)
            "李振藩",  # Bruce Lee
        }
    }

    places = {
        "en": {
            # "Shanghai",
            "Karachi",
            # "Beijing",
            "Delhi",
            "Lagos",
            "Tianjin",
            "Istanbul",
            # "Tokyo",
        },
        "ja": {
            "広島市",  # Hiroshima
            "京都市",  # Kyoto
            "富士山",  # Mount Fuji
            "長崎市",  # Nagasaki
            "大阪市",  # Osaka
            "東京",  # Tokyo
        },
        "ru": {
            "Москва",  # Moscow
            "Красная площадь",  # Red Square
            "Собор Василия Блаженного",  # Saint Basil's Cathedral
            "Санкт-Петербург",  # Saint Petersburg
            "Зимний дворец",  # Winter Palace
        },
        "zh": {
            "北京市",  # Beijing
            "上海市",  # Shanghai
            "香港特别行政区",  # Hong Kong
            "万里长城",  # Great Wall
            "故宮博物院",  # Forbidden City
        }
    }

    things = {
        "en": {
            "A-bomb",
            "Big Bang",
            "Encyclopédie",
            "Genetics",
            "Magna Carta",
            "Quaternion",
        },
        "ja": {
            "すし",  # sushi
            "歌舞伎",  # kabuki
            "浮世絵",  # ukiyo-e
            "相撲",  # sumo
            "暗黒舞踏",  # butoh
        },
        "ru": {
            "Балалайка",  # balalaika
            "Коммунистическая партия Советского Союза",  # communist party
            "Автомат Калашникова",  # AK-47
            "пирожки",  # pirozhki
            "водка",  # vodka
        },
        "zh": {
            "功夫",  # kung fu
            "中國園林",  # Chinese garden
            "舞獅",  # lion dance
            "道",  # Daoism
            "龍",  # dragon
        },
    }


def language():
    """
    returns random language code from selections
    """
    return random.choice(list(WPToolsTestTitles.featured))


def media(lang='en'):
    """
    returns random media title
    """
    titles = set()
    titles = titles.union(WPToolsTestTitles.books[lang])
    titles = titles.union(WPToolsTestTitles.music[lang])
    titles = titles.union(WPToolsTestTitles.film[lang])
    return {'lang': lang, 'title': random.choice(list(titles))}


def title(lang=None):
    """
    returns random title from selections
    """
    if not lang:
        lang = language()

    titles = set()
    if lang == 'en':
        titles = WPToolsTestTitles.animals['en']

    titles = titles.union(WPToolsTestTitles.featured[lang])
    titles = titles.union(WPToolsTestTitles.people[lang])
    titles = titles.union(WPToolsTestTitles.places[lang])
    titles = titles.union(WPToolsTestTitles.things[lang])

    return {'lang': lang, 'title': random.choice(list(titles))}



#!/usr/bin/env python

from setuptools import setup, find_packages

with open('README.rst', encoding='utf8') as f:
    readme = f.read()
with open('HISTORY.rst', encoding='utf8') as f:
    history = f.read()


setup(
    name='wptools_clone',
    version='0.4.20',
    description='Wikipedia tools (for Humans)',
    long_description=readme + '\n\n' + history,
    url='https://github.com/ider-zh/wptools',
    license='MIT',
    author='Steve @siznax',
    author_email='steve@siznax.net',
    py_modules=['wptools'],
    packages=find_packages(exclude=['tests']),
    test_suite='tests.test_basic',
    install_requires=['certifi', 'html2text', 'lxml'],
    include_package_data=True,
    entry_points={
        'console_scripts': ['wptool=wptools.wptool:main'],
    },
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
    ],
)



# -*- coding:utf-8 -*-

query = 'https://en.wikipedia.org/w/api.php?action=query&formatversion=2&list=categorymembers&cmlimit=500&cmtitle=Category%3ABAFTA%20winners%20%28people%29'

response = r"""{
    "batchcomplete": true,
    "continue": {
        "cmcontinue": "page|412745372f4907044b352f2f41273d03424b352f2f412704412745372f490121018f7f8f7f8f808f09|42525291",
        "continue": "-||"
    },
    "query": {
        "categorymembers": [
            {
                "pageid": 22167530,
                "ns": 0,
                "title": "Allison Abbate"
            }
        ]
    }
}"""

cache = {'query': query,
         'response': response,
         'info': {'content-type': 'TEST', 'status': 200}}



# -*- coding:utf-8 -*-

query = 'https://www.wikidata.org/w/api.php?action=wbgetentities&format=json&formatversion=2&ids=Q857243&languages=en&props=info|claims|descriptions|labels|sitelinks&redirects=yes&sites=&titles='

response = r"""
{
    "entities": {
        "Q857243": {
            "claims": {
                "P18": [
                    {
                        "mainsnak": {
                            "snaktype": "value",
                            "property": "P18",
                            "datavalue": {
                                "value": "Gujo Gifu Japan.jpg",
                                "type": "string"
                            },
                            "datatype": "commonsMedia"
                        },
                        "type": "statement",
                        "id": "q857243$50FA69EA-DC47-4C86-8748-FA11095C8A4F",
                        "rank": "normal"
                    },
                    {
                        "mainsnak": {
                            "snaktype": "value",
                            "property": "P18",
                            "datavalue": {
                                "value": "Gujo in Gifu Prefecture.png",
                                "type": "string"
                            },
                            "datatype": "commonsMedia"
                        },
                        "type": "statement",
                        "id": "q857243$8C06CEEB-F4D0-471F-95E9-90662320897A",
                        "rank": "normal"
                    },
                    {
                        "mainsnak": {
                            "snaktype": "value",
                            "property": "P18",
                            "datavalue": {
                                "value": "Flag of Gujo Gifu.JPG",
                                "type": "string"
                            },
                            "datatype": "commonsMedia"
                        },
                        "type": "statement",
                        "id": "q857243$510BBCCE-C86A-4E0D-99F6-36ACEC1B39FC",
                        "rank": "normal"
                    }
                ]
            }
        }
    },
    "success": 1
}
"""

cache = {'query': query, 'response': response}



# -*- coding:utf-8 -*-

query = 'https://en.wikipedia.org/w/api.php?action=query&exintro&formatversion=2&inprop=url|watchers&list=random&pithumbsize=240&pllimit=500&ppprop=disambiguation|wikibase_item&prop=extracts|info|links|pageassessments|pageimages|pageprops|pageterms|redirects&redirects&rdlimit=500&rnlimit=1&rnnamespace=0&titles=Douglas%20Adams'

response = r"""
{
    "batchcomplete": true,
    "continue": {
        "rncontinue": "0.767230557530|0.767230621808|8539703|0",
        "continue": "-||extracts|info|links|pageassessments|pageimages|pageprops|pageterms|redirects"
    },
    "warnings": {
        "extracts": {
            "warnings": "HTML may be malformed and/or unbalanced and may omit inline images. Use at your own risk. Known problems are listed at https://www.mediawiki.org/wiki/Extension:TextExtracts#Caveats."
        }
    },
    "query": {
        "pages": [
            {
                "pageid": 8091,
                "ns": 0,
                "title": "Douglas Adams",
                "extract": "<p><b>Douglas Noel Adams</b> (11 March 1952 – 11 May 2001) was an English author, scriptwriter, essayist, humorist, satirist and dramatist.</p>\n<p>Adams was author of <i>The Hitchhiker's Guide to the Galaxy</i>, which originated in 1978 as a BBC radio comedy before developing into a \"trilogy\" of five books that sold more than 15 million copies in his lifetime and generated a television series, several stage plays, comics, a computer game, and in 2005 a feature film. Adams's contribution to UK radio is commemorated in The Radio Academy's Hall of Fame.</p>\n<p>Adams also wrote <i>Dirk Gently's Holistic Detective Agency</i> (1987) and <i>The Long Dark Tea-Time of the Soul</i> (1988), and co-wrote <i>The Meaning of Liff</i> (1983), <i>The Deeper Meaning of Liff</i> (1990), <i>Last Chance to See</i> (1990), and three stories for the television series <i>Doctor Who</i>; he also served as script editor for the show's seventeenth season in 1979. A posthumous collection of his works, including an unfinished novel, was published as <i>The Salmon of Doubt</i> in 2002.</p>\n<p>Adams was an advocate for environmentalism and conservation, a lover of fast cars, technological innovation and the Apple Macintosh, and a radical atheist.</p>\n<p></p>",
                "contentmodel": "wikitext",
                "pagelanguage": "en",
                "pagelanguagehtmlcode": "en",
                "pagelanguagedir": "ltr",
                "touched": "2018-05-22T17:21:30Z",
                "lastrevid": 841482403,
                "length": 62123,
                "watchers": 460,
                "fullurl": "https://en.wikipedia.org/wiki/Douglas_Adams",
                "editurl": "https://en.wikipedia.org/w/index.php?title=Douglas_Adams&action=edit",
                "canonicalurl": "https://en.wikipedia.org/wiki/Douglas_Adams",
                "links": [
                    {
                        "ns": 0,
                        "title": ".Mac"
                    },
                    {
                        "ns": 0,
                        "title": "18610 Arthurdent"
                    },
                    {
                        "ns": 0,
                        "title": "25924 Douglasadams"
                    },
                    {
                        "ns": 0,
                        "title": "42 Puzzle"
                    },
                    {
                        "ns": 0,
                        "title": "69,105"
                    },
                    {
                        "ns": 0,
                        "title": "A Mind Forever Voyaging"
                    },
                    {
                        "ns": 0,
                        "title": "Academic conference"
                    },
                    {
                        "ns": 0,
                        "title": "Activision"
                    },
                    {
                        "ns": 0,
                        "title": "Albert Vezza"
                    },
                    {
                        "ns": 0,
                        "title": "American Atheists"
                    },
                    {
                        "ns": 0,
                        "title": "Amy Briggs"
                    },
                    {
                        "ns": 0,
                        "title": "And Another Thing... (novel)"
                    },
                    {
                        "ns": 0,
                        "title": "Anthony Read"
                    },
                    {
                        "ns": 0,
                        "title": "AppleMasters"
                    },
                    {
                        "ns": 0,
                        "title": "Apricot Computers"
                    },
                    {
                        "ns": 0,
                        "title": "Archive on 4"
                    },
                    {
                        "ns": 0,
                        "title": "Artemis Fowl (series)"
                    },
                    {
                        "ns": 0,
                        "title": "Arthur: The Quest for Excalibur"
                    },
                    {
                        "ns": 0,
                        "title": "Arthur Dent"
                    },
                    {
                        "ns": 0,
                        "title": "Atheism"
                    },
                    {
                        "ns": 0,
                        "title": "Atheist"
                    },
                    {
                        "ns": 0,
                        "title": "Audiobook"
                    },
                    {
                        "ns": 0,
                        "title": "Author"
                    },
                    {
                        "ns": 0,
                        "title": "BAFTA"
                    },
                    {
                        "ns": 0,
                        "title": "BBC"
                    },
                    {
                        "ns": 0,
                        "title": "BBC7"
                    },
                    {
                        "ns": 0,
                        "title": "BBC Micro"
                    },
                    {
                        "ns": 0,
                        "title": "BBC One"
                    },
                    {
                        "ns": 0,
                        "title": "BBC Online"
                    },
                    {
                        "ns": 0,
                        "title": "BBC Radio 4"
                    },
                    {
                        "ns": 0,
                        "title": "BBC Radio Four"
                    },
                    {
                        "ns": 0,
                        "title": "BBC Two"
                    },
                    {
                        "ns": 0,
                        "title": "BIBSYS"
                    },
                    {
                        "ns": 0,
                        "title": "Ballyhoo (video game)"
                    },
                    {
                        "ns": 0,
                        "title": "BattleTech: The Crescent Hawk's Inception"
                    },
                    {
                        "ns": 0,
                        "title": "BattleTech: The Crescent Hawk's Revenge"
                    },
                    {
                        "ns": 0,
                        "title": "Beyond Zork"
                    },
                    {
                        "ns": 0,
                        "title": "Biblioteca Nacional de España"
                    },
                    {
                        "ns": 0,
                        "title": "Bibliothèque nationale de France"
                    },
                    {
                        "ns": 0,
                        "title": "Bidenichthys beeblebroxi"
                    },
                    {
                        "ns": 0,
                        "title": "Big Finish Productions"
                    },
                    {
                        "ns": 0,
                        "title": "Black Cinderella Two Goes East"
                    },
                    {
                        "ns": 0,
                        "title": "Black rhino"
                    },
                    {
                        "ns": 0,
                        "title": "Black rhinoceros"
                    },
                    {
                        "ns": 0,
                        "title": "Blue plaque"
                    },
                    {
                        "ns": 0,
                        "title": "Bob Bates"
                    },
                    {
                        "ns": 0,
                        "title": "Book at Bedtime"
                    },
                    {
                        "ns": 0,
                        "title": "Border Zone (video game)"
                    },
                    {
                        "ns": 0,
                        "title": "Brain Damage (song)"
                    },
                    {
                        "ns": 0,
                        "title": "Brazil"
                    },
                    {
                        "ns": 0,
                        "title": "Brentwood, Essex"
                    },
                    {
                        "ns": 0,
                        "title": "Brentwood School (England)"
                    },
                    {
                        "ns": 0,
                        "title": "Brentwood School (Essex)"
                    },
                    {
                        "ns": 0,
                        "title": "Brian Moriarty"
                    },
                    {
                        "ns": 0,
                        "title": "Bruce Daniels"
                    },
                    {
                        "ns": 0,
                        "title": "Bureaucracy (computer game)"
                    },
                    {
                        "ns": 0,
                        "title": "Bureaucracy (video game)"
                    },
                    {
                        "ns": 0,
                        "title": "Cambridge"
                    },
                    {
                        "ns": 0,
                        "title": "Cambridge Z88"
                    },
                    {
                        "ns": 0,
                        "title": "Channel9.msdn.com"
                    },
                    {
                        "ns": 0,
                        "title": "Channel 9 (discussion forum)"
                    },
                    {
                        "ns": 0,
                        "title": "Charles Thomson (artist)"
                    },
                    {
                        "ns": 0,
                        "title": "Chinese river dolphin"
                    },
                    {
                        "ns": 0,
                        "title": "Christopher Cerf (musician and television producer)"
                    },
                    {
                        "ns": 0,
                        "title": "Christopher H. Bidmead"
                    },
                    {
                        "ns": 0,
                        "title": "CiNii"
                    },
                    {
                        "ns": 0,
                        "title": "Circuit's Edge"
                    },
                    {
                        "ns": 0,
                        "title": "City of Death"
                    },
                    {
                        "ns": 0,
                        "title": "Classic Text Adventure Masterpieces of Infocom"
                    },
                    {
                        "ns": 0,
                        "title": "Cocoa (API)"
                    },
                    {
                        "ns": 0,
                        "title": "Codie"
                    },
                    {
                        "ns": 0,
                        "title": "Collaborative writing"
                    },
                    {
                        "ns": 0,
                        "title": "Comedy"
                    },
                    {
                        "ns": 0,
                        "title": "Commodore PET"
                    },
                    {
                        "ns": 0,
                        "title": "Conservation movement"
                    },
                    {
                        "ns": 0,
                        "title": "Cornerstone (software)"
                    },
                    {
                        "ns": 0,
                        "title": "Cutthroats (video game)"
                    },
                    {
                        "ns": 0,
                        "title": "Dave Lebling"
                    },
                    {
                        "ns": 0,
                        "title": "David Agnew"
                    },
                    {
                        "ns": 0,
                        "title": "David Fisher (writer)"
                    },
                    {
                        "ns": 0,
                        "title": "David Gilmour"
                    },
                    {
                        "ns": 0,
                        "title": "David Irving"
                    },
                    {
                        "ns": 0,
                        "title": "Deadline (video game)"
                    },
                    {
                        "ns": 0,
                        "title": "Destiny of the Daleks"
                    },
                    {
                        "ns": 0,
                        "title": "Dian Fossey"
                    },
                    {
                        "ns": 0,
                        "title": "Digital Equipment Corporation"
                    },
                    {
                        "ns": 0,
                        "title": "Dirk (play)"
                    },
                    {
                        "ns": 0,
                        "title": "Dirk Gently"
                    },
                    {
                        "ns": 0,
                        "title": "Dirk Gently's Holistic Detective Agency"
                    },
                    {
                        "ns": 0,
                        "title": "Dirk Gently's Holistic Detective Agency (TV series)"
                    },
                    {
                        "ns": 0,
                        "title": "Dirk Gently's Holistic Detective Agency (radio serial)"
                    },
                    {
                        "ns": 0,
                        "title": "Dirk Gently (TV series)"
                    },
                    {
                        "ns": 0,
                        "title": "Dirk Maggs"
                    },
                    {
                        "ns": 0,
                        "title": "Disney"
                    },
                    {
                        "ns": 0,
                        "title": "Doctor Snuggles"
                    },
                    {
                        "ns": 0,
                        "title": "Doctor Who"
                    },
                    {
                        "ns": 0,
                        "title": "Doctor Who (season 16)"
                    },
                    {
                        "ns": 0,
                        "title": "Doctor Who spin-offs"
                    },
                    {
                        "ns": 0,
                        "title": "Doctor in the House (TV series)"
                    },
                    {
                        "ns": 0,
                        "title": "Doctor on the Go"
                    },
                    {
                        "ns": 0,
                        "title": "Don't Panic: The Official Hitchhiker's Guide to the Galaxy Companion"
                    },
                    {
                        "ns": 0,
                        "title": "Douglas Adams's Guide to The Hitchhiker's Guide to the Galaxy"
                    },
                    {
                        "ns": 0,
                        "title": "Douglas Adams's Starship Titanic"
                    },
                    {
                        "ns": 0,
                        "title": "Douglas Adams (disambiguation)"
                    },
                    {
                        "ns": 0,
                        "title": "Douglas Adams at the BBC"
                    },
                    {
                        "ns": 0,
                        "title": "Dramatist"
                    },
                    {
                        "ns": 0,
                        "title": "E-book"
                    },
                    {
                        "ns": 0,
                        "title": "Eagle (comic)"
                    },
                    {
                        "ns": 0,
                        "title": "Early adopter"
                    },
                    {
                        "ns": 0,
                        "title": "East End of London"
                    },
                    {
                        "ns": 0,
                        "title": "Eclipse (song)"
                    },
                    {
                        "ns": 0,
                        "title": "Edinburgh Fringe"
                    },
                    {
                        "ns": 0,
                        "title": "Embedded system"
                    },
                    {
                        "ns": 0,
                        "title": "Enchanter (video game)"
                    },
                    {
                        "ns": 0,
                        "title": "Encyclopedia Galactica"
                    },
                    {
                        "ns": 0,
                        "title": "Endangered species"
                    },
                    {
                        "ns": 0,
                        "title": "English literature"
                    },
                    {
                        "ns": 0,
                        "title": "Environmental activist"
                    },
                    {
                        "ns": 0,
                        "title": "Eoin Colfer"
                    },
                    {
                        "ns": 0,
                        "title": "Erechthias beeblebroxi"
                    },
                    {
                        "ns": 0,
                        "title": "Essayist"
                    },
                    {
                        "ns": 0,
                        "title": "Exhibition (scholarship)"
                    },
                    {
                        "ns": 0,
                        "title": "Find a Grave"
                    },
                    {
                        "ns": 0,
                        "title": "Fine-tuned Universe"
                    },
                    {
                        "ns": 0,
                        "title": "Fooblitzky"
                    },
                    {
                        "ns": 0,
                        "title": "Footlights"
                    },
                    {
                        "ns": 0,
                        "title": "Ford Prefect (character)"
                    },
                    {
                        "ns": 0,
                        "title": "Gareth Roberts (writer)"
                    },
                    {
                        "ns": 0,
                        "title": "Geoffrey Perkins"
                    },
                    {
                        "ns": 0,
                        "title": "Get Lamp"
                    },
                    {
                        "ns": 0,
                        "title": "Google Doodle"
                    },
                    {
                        "ns": 0,
                        "title": "Graham Chapman"
                    },
                    {
                        "ns": 0,
                        "title": "Graham Williams (television producer)"
                    },
                    {
                        "ns": 0,
                        "title": "Great Ape Project"
                    },
                    {
                        "ns": 0,
                        "title": "Gregory Hines"
                    },
                    {
                        "ns": 0,
                        "title": "Griff Rhys Jones"
                    },
                    {
                        "ns": 0,
                        "title": "Grue (monster)"
                    },
                    {
                        "ns": 0,
                        "title": "H2g2"
                    },
                    {
                        "ns": 0,
                        "title": "Harmony Books"
                    },
                    {
                        "ns": 0,
                        "title": "Harvey Mudd College"
                    },
                    {
                        "ns": 0,
                        "title": "Hello, sailor"
                    },
                    {
                        "ns": 0,
                        "title": "Highgate Cemetery"
                    },
                    {
                        "ns": 0,
                        "title": "Hitch-hiker's Guide to Europe"
                    },
                    {
                        "ns": 0,
                        "title": "Hitchcon"
                    },
                    {
                        "ns": 0,
                        "title": "Hitchhiker: A Biography of Douglas Adams"
                    },
                    {
                        "ns": 0,
                        "title": "Hollywood Hijinx"
                    },
                    {
                        "ns": 0,
                        "title": "Hugo Award"
                    },
                    {
                        "ns": 0,
                        "title": "Hugo Award for Best Dramatic Presentation"
                    },
                    {
                        "ns": 0,
                        "title": "Hyperland"
                    },
                    {
                        "ns": 0,
                        "title": "Hypertext"
                    },
                    {
                        "ns": 0,
                        "title": "IMDb"
                    },
                    {
                        "ns": 0,
                        "title": "IMovie"
                    },
                    {
                        "ns": 0,
                        "title": "ITV (TV network)"
                    },
                    {
                        "ns": 0,
                        "title": "Implementer (video games)"
                    },
                    {
                        "ns": 0,
                        "title": "Infidel (video game)"
                    },
                    {
                        "ns": 0,
                        "title": "InfoTaskForce"
                    },
                    {
                        "ns": 0,
                        "title": "Infocom"
                    },
                    {
                        "ns": 0,
                        "title": "Innovation"
                    },
                    {
                        "ns": 0,
                        "title": "Innsbruck"
                    },
                    {
                        "ns": 0,
                        "title": "Integrated Authority File"
                    },
                    {
                        "ns": 0,
                        "title": "Interactive fiction"
                    },
                    {
                        "ns": 0,
                        "title": "International Standard Book Number"
                    },
                    {
                        "ns": 0,
                        "title": "International Standard Name Identifier"
                    },
                    {
                        "ns": 0,
                        "title": "International Standard Serial Number"
                    },
                    {
                        "ns": 0,
                        "title": "InvisiClues"
                    },
                    {
                        "ns": 0,
                        "title": "Islington"
                    },
                    {
                        "ns": 0,
                        "title": "Istituto Centrale per il Catalogo Unico"
                    },
                    {
                        "ns": 0,
                        "title": "Jack Straw"
                    },
                    {
                        "ns": 0,
                        "title": "James Clavell's Shōgun"
                    },
                    {
                        "ns": 0,
                        "title": "James Goss (producer)"
                    },
                    {
                        "ns": 0,
                        "title": "Joe Ybarra"
                    },
                    {
                        "ns": 0,
                        "title": "John Cleese"
                    },
                    {
                        "ns": 0,
                        "title": "John Lloyd (producer)"
                    },
                    {
                        "ns": 0,
                        "title": "Journey (1989 video game)"
                    },
                    {
                        "ns": 0,
                        "title": "Justin, Charles & Co."
                    },
                    {
                        "ns": 0,
                        "title": "Kakapo"
                    },
                    {
                        "ns": 0,
                        "title": "Karey Kirkpatrick"
                    },
                    {
                        "ns": 0,
                        "title": "Keith Topping"
                    },
                    {
                        "ns": 0,
                        "title": "Kim Fuller"
                    },
                    {
                        "ns": 0,
                        "title": "Knowledge Navigator"
                    },
                    {
                        "ns": 0,
                        "title": "LIBRIS"
                    },
                    {
                        "ns": 0,
                        "title": "Labyrinth: The Computer Game"
                    },
                    {
                        "ns": 0,
                        "title": "Last Chance to See"
                    },
                    {
                        "ns": 0,
                        "title": "Leather Goddesses of Phobos"
                    },
                    {
                        "ns": 0,
                        "title": "Leather Goddesses of Phobos 2: Gas Pump Girls Meet the Pulsating Inconvenience from Planet X!"
                    },
                    {
                        "ns": 0,
                        "title": "Legend Entertainment"
                    },
                    {
                        "ns": 0,
                        "title": "Legends of Zork"
                    },
                    {
                        "ns": 0,
                        "title": "Library of Congress Control Number"
                    },
                    {
                        "ns": 0,
                        "title": "Life, the Universe and Everything"
                    },
                    {
                        "ns": 0,
                        "title": "List of Doctor Who episodes (1963–1989)"
                    },
                    {
                        "ns": 0,
                        "title": "List of Monty Python's Flying Circus episodes"
                    },
                    {
                        "ns": 0,
                        "title": "List of humorists"
                    },
                    {
                        "ns": 0,
                        "title": "List of minor The Hitchhiker's Guide to the Galaxy characters"
                    },
                    {
                        "ns": 0,
                        "title": "List of races and species in The Hitchhiker's Guide to the Galaxy"
                    },
                    {
                        "ns": 0,
                        "title": "List of recurring Monty Python's Flying Circus characters"
                    },
                    {
                        "ns": 0,
                        "title": "Lucasfilm Games"
                    },
                    {
                        "ns": 0,
                        "title": "MacUser"
                    },
                    {
                        "ns": 0,
                        "title": "Mac OS X"
                    },
                    {
                        "ns": 0,
                        "title": "Macintosh"
                    },
                    {
                        "ns": 0,
                        "title": "Marc Blank"
                    },
                    {
                        "ns": 0,
                        "title": "Mark Carwardine"
                    },
                    {
                        "ns": 0,
                        "title": "Martin Day"
                    },
                    {
                        "ns": 0,
                        "title": "Marvin the Paranoid Android"
                    },
                    {
                        "ns": 0,
                        "title": "Michael Berlyn"
                    },
                    {
                        "ns": 0,
                        "title": "Michael Palin"
                    },
                    {
                        "ns": 0,
                        "title": "Microsoft"
                    },
                    {
                        "ns": 0,
                        "title": "Mines of Titan"
                    },
                    {
                        "ns": 0,
                        "title": "Minor Planet Center"
                    },
                    {
                        "ns": 0,
                        "title": "Minor Planet Circular"
                    },
                    {
                        "ns": 0,
                        "title": "Montecito, California"
                    },
                    {
                        "ns": 0,
                        "title": "Monty Python"
                    },
                    {
                        "ns": 0,
                        "title": "Monty Python's Flying Circus"
                    },
                    {
                        "ns": 0,
                        "title": "Moonmist"
                    },
                    {
                        "ns": 0,
                        "title": "Mostly Harmless"
                    },
                    {
                        "ns": 0,
                        "title": "Mount Kilimanjaro"
                    },
                    {
                        "ns": 0,
                        "title": "Mountain gorilla"
                    },
                    {
                        "ns": 0,
                        "title": "MusicBrainz"
                    },
                    {
                        "ns": 0,
                        "title": "Myocardial infarction"
                    },
                    {
                        "ns": 0,
                        "title": "National Diet Library"
                    },
                    {
                        "ns": 0,
                        "title": "National Library of Australia"
                    },
                    {
                        "ns": 0,
                        "title": "National Library of the Czech Republic"
                    },
                    {
                        "ns": 0,
                        "title": "National Public Radio"
                    },
                    {
                        "ns": 0,
                        "title": "Natural history"
                    },
                    {
                        "ns": 0,
                        "title": "Neil Gaiman"
                    },
                    {
                        "ns": 0,
                        "title": "Neil Innes"
                    },
                    {
                        "ns": 0,
                        "title": "Noel Edmonds"
                    },
                    {
                        "ns": 0,
                        "title": "Nord and Bert Couldn't Make Head or Tail of It"
                    },
                    {
                        "ns": 0,
                        "title": "Northern white rhinoceros"
                    },
                    {
                        "ns": 0,
                        "title": "Not the Nine O'Clock News"
                    },
                    {
                        "ns": 0,
                        "title": "One of These Nights"
                    },
                    {
                        "ns": 0,
                        "title": "Out of the Trees"
                    },
                    {
                        "ns": 0,
                        "title": "Paola Cavalieri"
                    },
                    {
                        "ns": 0,
                        "title": "Patient Abuse"
                    },
                    {
                        "ns": 0,
                        "title": "Paul Cornell"
                    },
                    {
                        "ns": 0,
                        "title": "Paul McCartney"
                    },
                    {
                        "ns": 0,
                        "title": "Paul McGann"
                    },
                    {
                        "ns": 0,
                        "title": "Paul Neil Milne Johnstone"
                    },
                    {
                        "ns": 0,
                        "title": "Paul Wickens"
                    },
                    {
                        "ns": 0,
                        "title": "Peter Fincham"
                    },
                    {
                        "ns": 0,
                        "title": "Peter Singer"
                    },
                    {
                        "ns": 0,
                        "title": "Peter Stothard"
                    },
                    {
                        "ns": 0,
                        "title": "Phrases from The Hitchhiker's Guide to the Galaxy"
                    },
                    {
                        "ns": 0,
                        "title": "Pilot (Dirk Gently)"
                    },
                    {
                        "ns": 0,
                        "title": "Pink Floyd"
                    },
                    {
                        "ns": 0,
                        "title": "Places in The Hitchhiker's Guide to the Galaxy"
                    },
                    {
                        "ns": 0,
                        "title": "Planetfall"
                    },
                    {
                        "ns": 0,
                        "title": "Plundered Hearts"
                    },
                    {
                        "ns": 0,
                        "title": "Preparatory school (UK)"
                    },
                    {
                        "ns": 0,
                        "title": "Procol Harum"
                    },
                    {
                        "ns": 0,
                        "title": "Professional Developers Conference"
                    },
                    {
                        "ns": 0,
                        "title": "Professor Chronotis"
                    },
                    {
                        "ns": 0,
                        "title": "Quarterstaff: The Tomb of Setmoth"
                    },
                    {
                        "ns": 0,
                        "title": "RSPCA"
                    },
                    {
                        "ns": 0,
                        "title": "Radio Academy"
                    },
                    {
                        "ns": 0,
                        "title": "Rainbow 100"
                    },
                    {
                        "ns": 0,
                        "title": "Return to Zork"
                    },
                    {
                        "ns": 0,
                        "title": "Richard Dawkins"
                    },
                    {
                        "ns": 0,
                        "title": "Robin Day"
                    },
                    {
                        "ns": 0,
                        "title": "Russian State Library"
                    },
                    {
                        "ns": 0,
                        "title": "SNAC"
                    },
                    {
                        "ns": 0,
                        "title": "Sally Emerson"
                    },
                    {
                        "ns": 0,
                        "title": "Salon (website)"
                    },
                    {
                        "ns": 0,
                        "title": "Santa Barbara, California"
                    },
                    {
                        "ns": 0,
                        "title": "Santa Catarina (state)"
                    },
                    {
                        "ns": 0,
                        "title": "Satire"
                    },
                    {
                        "ns": 0,
                        "title": "Satirist"
                    },
                    {
                        "ns": 0,
                        "title": "Save the Rhino"
                    },
                    {
                        "ns": 0,
                        "title": "Science fiction"
                    },
                    {
                        "ns": 0,
                        "title": "Script editor"
                    },
                    {
                        "ns": 0,
                        "title": "Scriptwriter"
                    },
                    {
                        "ns": 0,
                        "title": "Seastalker"
                    },
                    {
                        "ns": 0,
                        "title": "Shada (Doctor Who)"
                    },
                    {
                        "ns": 0,
                        "title": "Sherlock: The Riddle of the Crown Jewels"
                    },
                    {
                        "ns": 0,
                        "title": "Sic"
                    },
                    {
                        "ns": 0,
                        "title": "Simon & Schuster"
                    },
                    {
                        "ns": 0,
                        "title": "Simon Brett"
                    },
                    {
                        "ns": 0,
                        "title": "Simon Jones (actor)"
                    },
                    {
                        "ns": 0,
                        "title": "Slartibartfast"
                    },
                    {
                        "ns": 0,
                        "title": "So Long, and Thanks for All the Fish"
                    },
                    {
                        "ns": 0,
                        "title": "Somebody else's problem"
                    },
                    {
                        "ns": 0,
                        "title": "Sorcerer (video game)"
                    },
                    {
                        "ns": 0,
                        "title": "Spellbreaker"
                    },
                    {
                        "ns": 0,
                        "title": "St. Cedd's College, Cambridge"
                    },
                    {
                        "ns": 0,
                        "title": "St John's College, Cambridge"
                    },
                    {
                        "ns": 0,
                        "title": "St Martin-in-the-Fields"
                    },
                    {
                        "ns": 0,
                        "title": "Starcross (video game)"
                    },
                    {
                        "ns": 0,
                        "title": "Starship Titanic"
                    },
                    {
                        "ns": 0,
                        "title": "Stationfall"
                    },
                    {
                        "ns": 0,
                        "title": "Stephen Fry"
                    },
                    {
                        "ns": 0,
                        "title": "Steve Meretzky"
                    },
                    {
                        "ns": 0,
                        "title": "Steven Moffat"
                    },
                    {
                        "ns": 0,
                        "title": "Strike action"
                    },
                    {
                        "ns": 0,
                        "title": "Stuckism"
                    },
                    {
                        "ns": 0,
                        "title": "Suspect (video game)"
                    },
                    {
                        "ns": 0,
                        "title": "Suspended (video game)"
                    },
                    {
                        "ns": 0,
                        "title": "Système universitaire de documentation"
                    },
                    {
                        "ns": 0,
                        "title": "São José, Santa Catarina"
                    },
                    {
                        "ns": 0,
                        "title": "TED (conference)"
                    },
                    {
                        "ns": 0,
                        "title": "Tandy 1000"
                    },
                    {
                        "ns": 0,
                        "title": "Tanzania"
                    },
                    {
                        "ns": 0,
                        "title": "Technology in The Hitchhiker's Guide to the Galaxy"
                    },
                    {
                        "ns": 0,
                        "title": "Ted Nelson"
                    },
                    {
                        "ns": 0,
                        "title": "Terry Jones"
                    },
                    {
                        "ns": 0,
                        "title": "The Album of the Soundtrack of the Trailer of the Film of Monty Python and the Holy Grail"
                    },
                    {
                        "ns": 0,
                        "title": "The Beatles"
                    },
                    {
                        "ns": 0,
                        "title": "The Burkiss Way"
                    },
                    {
                        "ns": 0,
                        "title": "The Daily Telegraph"
                    },
                    {
                        "ns": 0,
                        "title": "The Deeper Meaning of Liff"
                    },
                    {
                        "ns": 0,
                        "title": "The Digital Village"
                    },
                    {
                        "ns": 0,
                        "title": "The Discontinuity Guide"
                    },
                    {
                        "ns": 0,
                        "title": "The Division Bell"
                    },
                    {
                        "ns": 0,
                        "title": "The Doctor (Doctor Who)"
                    },
                    {
                        "ns": 0,
                        "title": "The God Delusion"
                    },
                    {
                        "ns": 0,
                        "title": "The Guardian"
                    },
                    {
                        "ns": 0,
                        "title": "The Hitchhiker's Guide to the Future"
                    },
                    {
                        "ns": 0,
                        "title": "The Hitchhiker's Guide to the Galaxy"
                    },
                    {
                        "ns": 0,
                        "title": "The Hitchhiker's Guide to the Galaxy: The Original Radio Scripts"
                    },
                    {
                        "ns": 0,
                        "title": "The Hitchhiker's Guide to the Galaxy (TV series)"
                    },
                    {
                        "ns": 0,
                        "title": "The Hitchhiker's Guide to the Galaxy (book)"
                    },
                    {
                        "ns": 0,
                        "title": "The Hitchhiker's Guide to the Galaxy (computer game)"
                    },
                    {
                        "ns": 0,
                        "title": "The Hitchhiker's Guide to the Galaxy (fictional)"
                    },
                    {
                        "ns": 0,
                        "title": "The Hitchhiker's Guide to the Galaxy (film)"
                    },
                    {
                        "ns": 0,
                        "title": "The Hitchhiker's Guide to the Galaxy (novel)"
                    },
                    {
                        "ns": 0,
                        "title": "The Hitchhiker's Guide to the Galaxy (radio series)"
                    },
                    {
                        "ns": 0,
                        "title": "The Hitchhiker's Guide to the Galaxy (video game)"
                    },
                    {
                        "ns": 0,
                        "title": "The Hitchhiker's Guide to the Galaxy Primary and Secondary Phases"
                    },
                    {
                        "ns": 0,
                        "title": "The Hitchhiker's Guide to the Galaxy Tertiary to Hexagonal Phases"
                    },
                    {
                        "ns": 0,
                        "title": "The Hitchhiker's Guide to the Galaxy Tertiary to Quintessential Phases"
                    },
                    {
                        "ns": 0,
                        "title": "The Hitchhiker's Guide to the Galaxy cast lists"
                    },
                    {
                        "ns": 0,
                        "title": "The Independent"
                    },
                    {
                        "ns": 0,
                        "title": "The Independent on Sunday"
                    },
                    {
                        "ns": 0,
                        "title": "The Key to Time"
                    },
                    {
                        "ns": 0,
                        "title": "The Long Dark Tea-Time of the Soul"
                    },
                    {
                        "ns": 0,
                        "title": "The Long Dark Tea-Time of the Soul (radio serial)"
                    },
                    {
                        "ns": 0,
                        "title": "The Lost Treasures of Infocom"
                    },
                    {
                        "ns": 0,
                        "title": "The Lurking Horror"
                    },
                    {
                        "ns": 0,
                        "title": "The Meaning of Liff"
                    },
                    {
                        "ns": 0,
                        "title": "The News Huddlines"
                    },
                    {
                        "ns": 0,
                        "title": "The Pirate Planet"
                    },
                    {
                        "ns": 0,
                        "title": "The Private Life of Genghis Khan"
                    },
                    {
                        "ns": 0,
                        "title": "The Restaurant at the End of the Universe"
                    },
                    {
                        "ns": 0,
                        "title": "The Salmon of Doubt"
                    },
                    {
                        "ns": 0,
                        "title": "The Snowmen"
                    },
                    {
                        "ns": 0,
                        "title": "The Times"
                    },
                    {
                        "ns": 0,
                        "title": "The Utterly Utterly Merry Comic Relief Christmas Book"
                    },
                    {
                        "ns": 0,
                        "title": "The Witness (1983 video game)"
                    },
                    {
                        "ns": 0,
                        "title": "Tim Anderson (programmer)"
                    },
                    {
                        "ns": 0,
                        "title": "Timeline of The Hitchhiker's Guide to the Galaxy versions"
                    },
                    {
                        "ns": 0,
                        "title": "Tom Baker"
                    },
                    {
                        "ns": 0,
                        "title": "Tombs & Treasure"
                    },
                    {
                        "ns": 0,
                        "title": "Towel Day"
                    },
                    {
                        "ns": 0,
                        "title": "Trafalgar Square"
                    },
                    {
                        "ns": 0,
                        "title": "Trillian (character)"
                    },
                    {
                        "ns": 0,
                        "title": "Trinity (video game)"
                    },
                    {
                        "ns": 0,
                        "title": "USENET"
                    },
                    {
                        "ns": 0,
                        "title": "Upper Street"
                    },
                    {
                        "ns": 0,
                        "title": "Virgin Books"
                    },
                    {
                        "ns": 0,
                        "title": "Virtual International Authority File"
                    },
                    {
                        "ns": 0,
                        "title": "Vogon"
                    },
                    {
                        "ns": 0,
                        "title": "Wayback Machine"
                    },
                    {
                        "ns": 0,
                        "title": "Webcast"
                    },
                    {
                        "ns": 0,
                        "title": "Week Ending"
                    },
                    {
                        "ns": 0,
                        "title": "West End of London"
                    },
                    {
                        "ns": 0,
                        "title": "Westwood Studios"
                    },
                    {
                        "ns": 0,
                        "title": "William Blake"
                    },
                    {
                        "ns": 0,
                        "title": "William Todd-Jones"
                    },
                    {
                        "ns": 0,
                        "title": "Wish You Were Here (1975 song)"
                    },
                    {
                        "ns": 0,
                        "title": "Wishbringer"
                    },
                    {
                        "ns": 0,
                        "title": "Word processor"
                    },
                    {
                        "ns": 0,
                        "title": "WorldCat"
                    },
                    {
                        "ns": 0,
                        "title": "Writer's block"
                    },
                    {
                        "ns": 0,
                        "title": "Young Zaphod Plays It Safe"
                    },
                    {
                        "ns": 0,
                        "title": "Z-machine"
                    },
                    {
                        "ns": 0,
                        "title": "Zaire"
                    },
                    {
                        "ns": 0,
                        "title": "Zaphod Beeblebrox"
                    },
                    {
                        "ns": 0,
                        "title": "Zork"
                    },
                    {
                        "ns": 0,
                        "title": "Zork: Grand Inquisitor"
                    },
                    {
                        "ns": 0,
                        "title": "Zork: The Undiscovered Underground"
                    },
                    {
                        "ns": 0,
                        "title": "Zork I"
                    },
                    {
                        "ns": 0,
                        "title": "Zork II"
                    },
                    {
                        "ns": 0,
                        "title": "Zork III"
                    },
                    {
                        "ns": 0,
                        "title": "Zork Nemesis"
                    },
                    {
                        "ns": 0,
                        "title": "Zork Zero"
                    },
                    {
                        "ns": 0,
                        "title": "Zork books"
                    },
                    {
                        "ns": 4,
                        "title": "Wikipedia:LIBRARY"
                    },
                    {
                        "ns": 10,
                        "title": "Template:Dirk Gently"
                    },
                    {
                        "ns": 10,
                        "title": "Template:Douglas Adams"
                    },
                    {
                        "ns": 10,
                        "title": "Template:Infocom games"
                    },
                    {
                        "ns": 10,
                        "title": "Template:The Hitchhiker's Guide to the Galaxy"
                    },
                    {
                        "ns": 11,
                        "title": "Template talk:Dirk Gently"
                    },
                    {
                        "ns": 11,
                        "title": "Template talk:Douglas Adams"
                    },
                    {
                        "ns": 11,
                        "title": "Template talk:Infocom games"
                    },
                    {
                        "ns": 11,
                        "title": "Template talk:The Hitchhiker's Guide to the Galaxy"
                    },
                    {
                        "ns": 12,
                        "title": "Help:Authority control"
                    },
                    {
                        "ns": 14,
                        "title": "Category:CS1 maint: Extra text: authors list"
                    },
                    {
                        "ns": 14,
                        "title": "Category:CS1 maint: Extra text: editors list"
                    },
                    {
                        "ns": 14,
                        "title": "Category:CS1 maint: Multiple names: authors list"
                    },
                    {
                        "ns": 14,
                        "title": "Category:The Hitchhiker's Guide to the Galaxy"
                    },
                    {
                        "ns": 14,
                        "title": "Category:Use British English from October 2013"
                    },
                    {
                        "ns": 14,
                        "title": "Category:Use dmy dates from May 2018"
                    },
                    {
                        "ns": 100,
                        "title": "Portal:Hitchhiker's"
                    }
                ],
                "pageassessments": {
                    "Biography": {
                        "class": "B",
                        "importance": ""
                    },
                    "Wikipedia 1.0": {
                        "class": "C",
                        "importance": "Mid"
                    },
                    "Science Fiction": {
                        "class": "C",
                        "importance": "Mid"
                    },
                    "Comedy": {
                        "class": "B",
                        "importance": "Mid"
                    },
                    "London": {
                        "class": "B",
                        "importance": "Low"
                    },
                    "Atheism": {
                        "class": "B",
                        "importance": "Low"
                    },
                    "BBC": {
                        "class": "B",
                        "importance": "Low"
                    },
                    "Monty Python": {
                        "class": "B",
                        "importance": "Mid"
                    },
                    "Hitchhiker's Guide to the Galaxy": {
                        "class": "",
                        "importance": ""
                    },
                    "Doctor Who": {
                        "class": "B",
                        "importance": "Low"
                    }
                },
                "thumbnail": {
                    "source": "https://upload.wikimedia.org/wikipedia/commons/thumb/c/c0/Douglas_adams_portrait_cropped.jpg/207px-Douglas_adams_portrait_cropped.jpg",
                    "width": 207,
                    "height": 240
                },
                "pageimage": "Douglas_adams_portrait_cropped.jpg",
                "pageprops": {
                    "wikibase_item": "Q42"
                },
                "terms": {
                    "alias": [
                        "Douglas N. Adams",
                        "Douglas Noel Adams",
                        "Douglas Noël Adams"
                    ],
                    "description": [
                        "author"
                    ],
                    "label": [
                        "Douglas Adams"
                    ]
                },
                "redirects": [
                    {
                        "pageid": 421613,
                        "ns": 0,
                        "title": "Douglas Noël Adams"
                    },
                    {
                        "pageid": 442167,
                        "ns": 0,
                        "title": "Douglas Adams Biography"
                    },
                    {
                        "pageid": 555606,
                        "ns": 0,
                        "title": "Douglas Noel Adams"
                    },
                    {
                        "pageid": 697699,
                        "ns": 0,
                        "title": "Bop Ad"
                    },
                    {
                        "pageid": 742979,
                        "ns": 0,
                        "title": "Adams, Douglas"
                    },
                    {
                        "pageid": 799056,
                        "ns": 0,
                        "title": "Adams, Douglas Noel"
                    },
                    {
                        "pageid": 911959,
                        "ns": 0,
                        "title": "Douglas adams"
                    },
                    {
                        "pageid": 944266,
                        "ns": 0,
                        "title": "Douglass adams"
                    },
                    {
                        "pageid": 19511273,
                        "ns": 0,
                        "title": "Jane Belson"
                    },
                    {
                        "pageid": 31976318,
                        "ns": 0,
                        "title": "Douglas Adams bibliography"
                    },
                    {
                        "pageid": 35473253,
                        "ns": 0,
                        "title": "Adams, Douglas Noël"
                    },
                    {
                        "pageid": 41687224,
                        "ns": 0,
                        "title": "Adamsian"
                    }
                ]
            }
        ],
        "random": [
            {
                "id": 7614573,
                "ns": 0,
                "title": "Aidenbachstraße (Munich U-Bahn)"
            }
        ]
    }
}
"""

cache = {'info': {'status': 200}, 'query': query, 'response': response}



# -*- coding:utf-8 -*-

query = 'https://www.wikidata.org/w/api.php?action=wbgetentities&formatversion=2&languages=en&props=labels&redirects=yes&ids=Q6607|Q7758404|P271|P172|Q902712|P31|P409|Q84|Q578895|P1258|P2435|P244|Q350|P213|Q533697|P3430|P1670|P40|P1695|Q5|P2019|P268|P551|Q245068|P396|P906|P3222|Q909993|P734|P735|P1477|P1368|P140|Q18844224|P2163|P3417|Q9531|P3106|P4431|P69|Q3414212|P227|P949|P947|P1617|P19|P1196'

response = r"""{
    "entities": {
        "Q6607": {
            "type": "item",
            "id": "Q6607",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "guitar"
                }
            }
        },
        "Q7758404": {
            "type": "item",
            "id": "Q7758404",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "The Private Life of Genghis Khan"
                }
            }
        },
        "P271": {
            "type": "property",
            "datatype": "external-id",
            "id": "P271",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "CiNii author ID"
                }
            }
        },
        "P172": {
            "type": "property",
            "datatype": "wikibase-item",
            "id": "P172",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "ethnic group"
                }
            }
        },
        "Q902712": {
            "type": "item",
            "id": "Q902712",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "Dirk Gently's Holistic Detective Agency"
                }
            }
        },
        "P31": {
            "type": "property",
            "datatype": "wikibase-item",
            "id": "P31",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "instance of"
                }
            }
        },
        "P409": {
            "type": "property",
            "datatype": "external-id",
            "id": "P409",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "NLA (Australia) ID"
                }
            }
        },
        "Q84": {
            "type": "item",
            "id": "Q84",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "London"
                }
            }
        },
        "Q578895": {
            "type": "item",
            "id": "Q578895",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "The Restaurant at the End of the Universe"
                }
            }
        },
        "P1258": {
            "type": "property",
            "datatype": "external-id",
            "id": "P1258",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "Rotten Tomatoes ID"
                }
            }
        },
        "P2435": {
            "type": "property",
            "datatype": "external-id",
            "id": "P2435",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "PORT person ID"
                }
            }
        },
        "P244": {
            "type": "property",
            "datatype": "external-id",
            "id": "P244",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "Library of Congress authority ID"
                }
            }
        },
        "Q350": {
            "type": "item",
            "id": "Q350",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "Cambridge"
                }
            }
        },
        "P213": {
            "type": "property",
            "datatype": "external-id",
            "id": "P213",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "ISNI"
                }
            }
        },
        "Q533697": {
            "type": "item",
            "id": "Q533697",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "Highgate Cemetery"
                }
            }
        },
        "P3430": {
            "type": "property",
            "datatype": "external-id",
            "id": "P3430",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "SNAC Ark ID"
                }
            }
        },
        "P1670": {
            "type": "property",
            "datatype": "external-id",
            "id": "P1670",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "LAC ID"
                }
            }
        },
        "P40": {
            "type": "property",
            "datatype": "wikibase-item",
            "id": "P40",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "child"
                }
            }
        },
        "P1695": {
            "type": "property",
            "datatype": "external-id",
            "id": "P1695",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "NLP ID"
                }
            }
        },
        "Q5": {
            "type": "item",
            "id": "Q5",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "human"
                }
            }
        },
        "P2019": {
            "type": "property",
            "datatype": "external-id",
            "id": "P2019",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "AllMovie artist ID"
                }
            }
        },
        "P268": {
            "type": "property",
            "datatype": "external-id",
            "id": "P268",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "BnF ID"
                }
            }
        },
        "P551": {
            "type": "property",
            "datatype": "wikibase-item",
            "id": "P551",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "residence"
                }
            }
        },
        "Q245068": {
            "type": "item",
            "id": "Q245068",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "comedian"
                }
            }
        },
        "P396": {
            "type": "property",
            "datatype": "external-id",
            "id": "P396",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "SBN ID"
                }
            }
        },
        "P906": {
            "type": "property",
            "datatype": "external-id",
            "id": "P906",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "SELIBR"
                }
            }
        },
        "P3222": {
            "type": "property",
            "datatype": "external-id",
            "id": "P3222",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "NE.se ID"
                }
            }
        },
        "Q909993": {
            "type": "item",
            "id": "Q909993",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "Brentwood"
                }
            }
        },
        "P734": {
            "type": "property",
            "datatype": "wikibase-item",
            "id": "P734",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "family name"
                }
            }
        },
        "P735": {
            "type": "property",
            "datatype": "wikibase-item",
            "id": "P735",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "given name"
                }
            }
        },
        "P1477": {
            "type": "property",
            "datatype": "monolingualtext",
            "id": "P1477",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "birth name"
                }
            }
        },
        "P1368": {
            "type": "property",
            "datatype": "external-id",
            "id": "P1368",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "LNB ID"
                }
            }
        },
        "P140": {
            "type": "property",
            "datatype": "wikibase-item",
            "id": "P140",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "religion"
                }
            }
        },
        "Q18844224": {
            "type": "item",
            "id": "Q18844224",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "science fiction writer"
                }
            }
        },
        "P2163": {
            "type": "property",
            "datatype": "external-id",
            "id": "P2163",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "FAST ID"
                }
            }
        },
        "P3417": {
            "type": "property",
            "datatype": "external-id",
            "id": "P3417",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "Quora topic ID"
                }
            }
        },
        "Q9531": {
            "type": "item",
            "id": "Q9531",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "BBC"
                }
            }
        },
        "P3106": {
            "type": "property",
            "datatype": "external-id",
            "id": "P3106",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "Guardian topic ID"
                }
            }
        },
        "P4431": {
            "type": "property",
            "datatype": "external-id",
            "id": "P4431",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "Google Doodle"
                }
            }
        },
        "P69": {
            "type": "property",
            "datatype": "wikibase-item",
            "id": "P69",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "educated at"
                }
            }
        },
        "Q3414212": {
            "type": "item",
            "id": "Q3414212",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "Hugo Award for Best Dramatic Presentation"
                }
            }
        },
        "P227": {
            "type": "property",
            "datatype": "external-id",
            "id": "P227",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "GND ID"
                }
            }
        },
        "P949": {
            "type": "property",
            "datatype": "external-id",
            "id": "P949",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "National Library of Israel ID"
                }
            }
        },
        "P947": {
            "type": "property",
            "datatype": "external-id",
            "id": "P947",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "RSL ID (person)"
                }
            }
        },
        "P1617": {
            "type": "property",
            "datatype": "external-id",
            "id": "P1617",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "BBC Things ID"
                }
            }
        },
        "P19": {
            "type": "property",
            "datatype": "wikibase-item",
            "id": "P19",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "place of birth"
                }
            }
        },
        "P1196": {
            "type": "property",
            "datatype": "wikibase-item",
            "id": "P1196",
            "labels": {
                "en": {
                    "language": "en",
                    "value": "manner of death"
                }
            }
        }
    },
    "success": 1
}"""

cache = {'info': {'status': 200}, 'query': query, 'response': response}



# -*- coding:utf-8 -*-

query = 'https://en.wikipedia.org/w/api.php?action=query&formatversion=2&list=random&rnlimit=1&rnnamespace=0'

response = r"""{
    "batchcomplete": true,
    "continue": {
        "rncontinue": "0.409283800887|0.409283961861|13478206|0",
        "continue": "-||"
    },
    "query": {
        "random": [
            {
                "id": 123456789,
                "ns": 0,
                "title": "RANDOM TEST TITLE"
            }
        ]
    }
}
"""

cache = {'info': {'status': 200}, 'query': query, 'response': response}



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Stress-test Wptools
~~~~~~~~~~~~~~~~~~~

Endlessly GET page info from ALL Wikimedia APIs in MANY languages
"""

from __future__ import print_function

import argparse
import random
import sys
import time

import wptools

DEFAULT_DELAY = 3
DEFAULT_LANGUAGE = 'en'

# connection refused 'ge'
# wikidata fail 'zh-min-nan'
LANGUAGES = [
    'en', 'ceb', 'sv', 'de', 'nl', 'fr', 'ru', 'it', 'es', 'war',
    'pl', 'vi', 'ja', 'pt', 'zh', 'uk', 'fa', 'ca', 'ar', 'no', 'sh',
    'fi', 'hu', 'id', 'ko', 'cs', 'ro', 'sr', 'ms', 'tr', 'eu', 'eo',
    'bg', 'da', 'hy', 'min', 'kk', 'sk', 'he', 'lt', 'hr', 'ce', 'et',
    'sl', 'be', 'el', 'nn', 'uz', 'simple', 'la', 'az', 'ur', 'hi',
    'vo', 'th', 'ka', 'ta', 'cy', 'mk', 'mg', 'oc', 'tl', 'lv', 'ky',
    'bs', 'tt', 'new', 'tg', 'sq', 'te', 'pms', 'br', 'bn', 'ml',
    'ht', 'jv', 'ast', 'lb', 'mr', 'af', 'sco', 'pnb', 'ga', 'is',
    'cv', 'ba', 'azb', 'fy', 'su', 'sw', 'my', 'lmo', 'an', 'yo',
    'ne', 'gu', 'io', 'pa', 'nds', 'scn', 'bpy', 'als', 'bar', 'ku',
    'kn', 'ia', 'qu', 'ckb', 'mn', 'arz']


def languages():
    """
    Many sites not fully supporting APIs. See LANGUAGES.
    """
    return LANGUAGES

    # site = wptools.site()
    # site.get_sites()
    # sites = site.data['sites']
    # wps = [x for x in sites if 'wikipedia' in x]
    # return [x.split('/')[-1].split('.')[0] for x in wps]


def popular(lang):
    """
    returns list of most popular pages
    """
    site = wptools.site(silent=True)
    if lang:
        site.get_info("%s.wikipedia.org" % lang)
    else:
        site.get_info()
    return [x['title'] for x in site.data['mostviewed']]


def print_header(delay, lang, pages):
    """
    print header for stress test
    """
    langstr = lang
    if langstr is None:
        langstr = len(languages())

    pagestr = len(pages)
    if pagestr == 1:
        pagestr = "+"

    msg = []
    msg.append("WPTOOLS STRESS TEST")
    msg.append(time.asctime(time.gmtime()))
    msg.append("delay: %d lang: %s pages: %s" % (delay, langstr, pagestr))
    msgstr = " ".join(msg)

    header = [msgstr]
    header.append("=" * len(msgstr))
    header.append("Python " + sys.version)
    header.append('-' * len(msgstr))

    if len(pages) > 1:
        print("Getting top %s.wikipedia.org pages" % lang)
        for i, title in enumerate(pages[:10]):
            print(" %d. %s" % (i + 1, title))

    print("\n".join(header))


def main(args):
    """
    GET top pages or random pages forever
    """
    delay = args.delay
    lang = args.lang
    top = args.top

    start = int(time.time())

    pages = ['forever']
    if top:
        pages = popular(lang)

    print_header(delay, lang, pages)

    try:
        count = 0
        requests = 0
        elapsed = 0

        while len(pages) > 0:
            language = lang or random.choice(languages())
            if top and not lang:
                language = 'en'

            page = wptools.page(lang=language, silent=True)
            if top:
                page = wptools.page(pages.pop(0), lang=language, silent=True)

            page.get()

            preview = page.data.get('extext')
            if preview:
                preview = preview.strip().replace("\n", '')[:64]

            url = page.data.get('url')

            elapsed = int(time.time()) - start

            count += 1
            nrq = len(page.data.get('requests'))
            requests += nrq
            rps = float(0)
            if elapsed > 0:
                rps = float(requests) / elapsed
            frps = '{:.1f}'.format(rps)

            print("[%d] %d %s %s" % (count, nrq, frps, url))
            print("%s %s" % (page.data.get('wikibase'), preview))

            time.sleep(delay)

    except KeyboardInterrupt:
        print("Done. %d requests %d seconds" % (requests, elapsed))

    except:
        page.flags['silent'] = False
        page.show()
        print("EXCEPTION %d requests %d seconds" % (requests, elapsed))
        raise


def parse_args():
    """
    parse args for main()
    """
    argp = argparse.ArgumentParser()
    argp.add_argument('-d', '--delay', help='delay in seconds',
                      type=int, default=DEFAULT_DELAY)
    argp.add_argument('-l', '--lang', help='language code')
    argp.add_argument('-t', '--top', action='store_true', help='get top pages')
    return argp.parse_args()


if __name__ == "__main__":
    main(parse_args())
